@@ -1,20 +1,9 @@
 package Alien::Leptonica;
-$Alien::Leptonica::VERSION = '0.006';
+$Alien::Leptonica::VERSION = '0.007';
 use strict;
 use warnings;
 
-use parent qw(Alien::Base Exporter);
-our @EXPORT_OK = qw(Inline);
-
-sub Inline {
-	return unless $_[-1] eq 'C'; # Inline's error message is good
-	my $self = __PACKAGE__->new;
-	+{
-		LIBS => $self->libs,
-		INC => $self->cflags,
-		AUTO_INCLUDE => '#include "allheaders.h"',
-	};
-}
+use parent qw(Alien::Base);
 
 1;
 
@@ -28,7 +17,7 @@ Alien::Leptonica - Alien package for the Leptonica image processing library
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 Inline support
 
