@@ -226,7 +226,7 @@ l_int32  i, start, len, stop, mid;
             revbarstr = stringReverse(barstr);
             start = !strncmp(revbarstr, Code2of5[C25_START], 3);
             stop = !strncmp(&revbarstr[len - 5], Code2of5[C25_STOP], 5);
-            FREE(revbarstr);
+            LEPT_FREE(revbarstr);
             if (start && stop) {
                 *pvalid = 1;
                 if (preverse) *preverse = 1;
@@ -243,7 +243,7 @@ l_int32  i, start, len, stop, mid;
             revbarstr = stringReverse(barstr);
             start = !strncmp(revbarstr, CodeI2of5[CI25_START], 4);
             stop = !strncmp(&revbarstr[len - 3], CodeI2of5[CI25_STOP], 3);
-            FREE(revbarstr);
+            LEPT_FREE(revbarstr);
             if (start && stop) {
                 *pvalid = 1;
                 if (preverse) *preverse = 1;
@@ -260,7 +260,7 @@ l_int32  i, start, len, stop, mid;
             revbarstr = stringReverse(barstr);
             start = !strncmp(revbarstr, Code93[C93_START], 6);
             stop = !strncmp(&revbarstr[len - 7], Code93[C93_STOP], 6);
-            FREE(revbarstr);
+            LEPT_FREE(revbarstr);
             if (start && stop) {
                 *pvalid = 1;
                 if (preverse) *preverse = 1;
@@ -277,7 +277,7 @@ l_int32  i, start, len, stop, mid;
             revbarstr = stringReverse(barstr);
             start = !strncmp(revbarstr, Code39[C39_START], 9);
             stop = !strncmp(&revbarstr[len - 9], Code39[C39_STOP], 9);
-            FREE(revbarstr);
+            LEPT_FREE(revbarstr);
             if (start && stop) {
                 *pvalid = 1;
                 if (preverse) *preverse = 1;
@@ -300,7 +300,7 @@ l_int32  i, start, len, stop, mid;
                 start += !strncmp(revbarstr, Codabar[i], 7);
             for (i = 16; i <= 19; i++)
                 stop += !strncmp(&revbarstr[len - 7], Codabar[i], 7);
-            FREE(revbarstr);
+            LEPT_FREE(revbarstr);
             if (start && stop) {
                 *pvalid = 1;
                 if (preverse) *preverse = 1;
@@ -384,7 +384,7 @@ l_int32  valid, reverse, i, j, len, error, ndigits, start, found;
 
     error = FALSE;
     ndigits = (len - 11) / 10;
-    data = (char *)CALLOC(ndigits + 1, sizeof(char));
+    data = (char *)LEPT_CALLOC(ndigits + 1, sizeof(char));
     memset(code, 0, 10);
     for (i = 0; i < ndigits; i++) {
         start = 6 + 10 * i;
@@ -394,20 +394,20 @@ l_int32  valid, reverse, i, j, len, error, ndigits, start, found;
         if (debugflag)
             fprintf(stderr, "code: %s\n", code);
 
-	found = FALSE;
+        found = FALSE;
         for (j = 0; j < 10; j++) {
             if (!strcmp(code, Code2of5[j])) {
                 data[i] = 0x30 + j;
-		found = TRUE;
+                found = TRUE;
                 break;
             }
         }
-	if (!found) error = TRUE;
+        if (!found) error = TRUE;
     }
-    FREE(vbarstr);
+    LEPT_FREE(vbarstr);
 
     if (error) {
-        FREE(data);
+        LEPT_FREE(data);
         return (char *)ERROR_PTR("error in decoding", procName, NULL);
     }
 
@@ -460,7 +460,7 @@ l_int32  valid, reverse, i, j, len, error, npairs, start, found;
 
     error = FALSE;
     npairs = (len - 7) / 10;
-    data = (char *)CALLOC(2 * npairs + 1, sizeof(char));
+    data = (char *)LEPT_CALLOC(2 * npairs + 1, sizeof(char));
     memset(code1, 0, 6);
     memset(code2, 0, 6);
     for (i = 0; i < npairs; i++) {
@@ -473,29 +473,29 @@ l_int32  valid, reverse, i, j, len, error, npairs, start, found;
         if (debugflag)
             fprintf(stderr, "code1: %s, code2: %s\n", code1, code2);
 
-	found = FALSE;
+        found = FALSE;
         for (j = 0; j < 10; j++) {
             if (!strcmp(code1, CodeI2of5[j])) {
                 data[2 * i] = 0x30 + j;
-		found = TRUE;
+                found = TRUE;
                 break;
             }
         }
-	if (!found) error = TRUE;
-	found = FALSE;
+        if (!found) error = TRUE;
+        found = FALSE;
         for (j = 0; j < 10; j++) {
             if (!strcmp(code2, CodeI2of5[j])) {
                 data[2 * i + 1] = 0x30 + j;
-		found = TRUE;
+                found = TRUE;
                 break;
             }
         }
-	if (!found) error = TRUE;
+        if (!found) error = TRUE;
     }
-    FREE(vbarstr);
+    LEPT_FREE(vbarstr);
 
     if (error) {
-        FREE(data);
+        LEPT_FREE(data);
         return (char *)ERROR_PTR("error in decoding", procName, NULL);
     }
 
@@ -555,8 +555,8 @@ l_int32     *index;
 
         /* Decode the symbols */
     nsymb = (len - 13) / 6;
-    data = (char *)CALLOC(nsymb + 1, sizeof(char));
-    index = (l_int32 *)CALLOC(nsymb, sizeof(l_int32));
+    data = (char *)LEPT_CALLOC(nsymb + 1, sizeof(char));
+    index = (l_int32 *)LEPT_CALLOC(nsymb, sizeof(l_int32));
     memset(code, 0, 7);
     error = FALSE;
     for (i = 0; i < nsymb; i++) {
@@ -572,17 +572,17 @@ l_int32     *index;
             if (!strcmp(code, Code93[j])) {
                 data[i] = Code93Val[j];
                 index[i] = j;
-		found = TRUE;
+                found = TRUE;
                 break;
             }
         }
-	if (!found) error = TRUE;
+        if (!found) error = TRUE;
     }
-    FREE(vbarstr);
+    LEPT_FREE(vbarstr);
 
     if (error) {
-        FREE(index);
-        FREE(data);
+        LEPT_FREE(index);
+        LEPT_FREE(data);
         return (char *)ERROR_PTR("error in decoding", procName, NULL);
     }
 
@@ -614,7 +614,7 @@ l_int32     *index;
         /* Remove the two check codes from the output */
     data[nsymb - 2] = '\0';
 
-    FREE(index);
+    LEPT_FREE(index);
     return data;
 }
 
@@ -666,7 +666,7 @@ l_int32   valid, reverse, i, j, len, error, nsymb, start, found;
 
         /* Decode the symbols */
     nsymb = (len - 19) / 10;
-    data = (char *)CALLOC(nsymb + 1, sizeof(char));
+    data = (char *)LEPT_CALLOC(nsymb + 1, sizeof(char));
     memset(code, 0, 10);
     error = FALSE;
     for (i = 0; i < nsymb; i++) {
@@ -687,10 +687,10 @@ l_int32   valid, reverse, i, j, len, error, nsymb, start, found;
         }
         if (!found) error = TRUE;
     }
-    FREE(vbarstr);
+    LEPT_FREE(vbarstr);
 
     if (error) {
-        FREE(data);
+        LEPT_FREE(data);
         return (char *)ERROR_PTR("error in decoding", procName, NULL);
     }
 
@@ -746,7 +746,7 @@ l_int32   valid, reverse, i, j, len, error, nsymb, start, found;
 
         /* Decode the symbols */
     nsymb = (len - 15) / 8;
-    data = (char *)CALLOC(nsymb + 1, sizeof(char));
+    data = (char *)LEPT_CALLOC(nsymb + 1, sizeof(char));
     memset(code, 0, 8);
     error = FALSE;
     for (i = 0; i < nsymb; i++) {
@@ -767,10 +767,10 @@ l_int32   valid, reverse, i, j, len, error, nsymb, start, found;
         }
         if (!found) error = TRUE;
     }
-    FREE(vbarstr);
+    LEPT_FREE(vbarstr);
 
     if (error) {
-        FREE(data);
+        LEPT_FREE(data);
         return (char *)ERROR_PTR("error in decoding", procName, NULL);
     }
 
@@ -840,7 +840,7 @@ l_int32   valid, i, j, len, error, start, found, sum, checkdigit;
         vbarstr = stringNew(barstr);
 
         /* Decode the 12 symbols */
-    data = (char *)CALLOC(13, sizeof(char));
+    data = (char *)LEPT_CALLOC(13, sizeof(char));
     memset(code, 0, 5);
     error = FALSE;
     for (i = 0; i < 12; i++) {
@@ -858,16 +858,16 @@ l_int32   valid, i, j, len, error, start, found, sum, checkdigit;
         for (j = 0; j < 10; j++) {
             if (!strcmp(code, Upca[j])) {
                 data[i] = 0x30 + j;
-		found = TRUE;
+                found = TRUE;
                 break;
             }
         }
-	if (!found) error = TRUE;
+        if (!found) error = TRUE;
     }
-    FREE(vbarstr);
+    LEPT_FREE(vbarstr);
 
     if (error) {
-        FREE(data);
+        LEPT_FREE(data);
         return (char *)ERROR_PTR("error in decoding", procName, NULL);
     }
 
@@ -917,7 +917,7 @@ l_int32   valid, i, j, len, error, start, found, sum, checkdigit;
 static char *
 barcodeDecodeEan13(char    *barstr,
                    l_int32  first,
-		   l_int32  debugflag)
+                   l_int32  debugflag)
 {
 char     *data, *vbarstr;
 char      code[5];
@@ -929,8 +929,8 @@ l_int32   valid, i, j, len, error, start, found, sum, checkdigit;
         return (char *)ERROR_PTR("barstr not defined", procName, NULL);
 
         /* Verify format.  You can't tell the orientation by the start
-	 * and stop codes, but you can by the location of the digits.
-	 * Use the UPCA verifier for EAN 13 -- it is identical. */
+         * and stop codes, but you can by the location of the digits.
+         * Use the UPCA verifier for EAN 13 -- it is identical. */
     barcodeVerifyFormat(barstr, L_BF_UPCA, &valid, NULL);
     if (!valid)
         return (char *)ERROR_PTR("barstr not in EAN 13 format", procName, NULL);
@@ -958,7 +958,7 @@ l_int32   valid, i, j, len, error, start, found, sum, checkdigit;
         vbarstr = stringNew(barstr);
 
         /* Decode the 12 symbols */
-    data = (char *)CALLOC(13, sizeof(char));
+    data = (char *)LEPT_CALLOC(13, sizeof(char));
     memset(code, 0, 5);
     error = FALSE;
     for (i = 0; i < 12; i++) {
@@ -976,16 +976,16 @@ l_int32   valid, i, j, len, error, start, found, sum, checkdigit;
         for (j = 0; j < 10; j++) {
             if (!strcmp(code, Upca[j])) {
                 data[i] = 0x30 + j;
-		found = TRUE;
+                found = TRUE;
                 break;
             }
         }
-	if (!found) error = TRUE;
+        if (!found) error = TRUE;
     }
-    FREE(vbarstr);
+    LEPT_FREE(vbarstr);
 
     if (error) {
-        FREE(data);
+        LEPT_FREE(data);
         return (char *)ERROR_PTR("error in decoding", procName, NULL);
     }
 
