@@ -162,7 +162,7 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getBilinearXformCoeffs(ptad, ptas, &vc);
     pixd = pixBilinearSampled(pixs, vc, incolor);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
@@ -425,7 +425,7 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getBilinearXformCoeffs(ptad, ptas, &vc);
     pixd = pixBilinearColor(pixs, vc, colorval);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
@@ -528,7 +528,7 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getBilinearXformCoeffs(ptad, ptas, &vc);
     pixd = pixBilinearGray(pixs, vc, grayval);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
@@ -776,7 +776,7 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     if (!pvc)
         return ERROR_INT("&vc not defined", procName, 1);
 
-    if ((b = (l_float32 *)CALLOC(8, sizeof(l_float32))) == NULL)
+    if ((b = (l_float32 *)LEPT_CALLOC(8, sizeof(l_float32))) == NULL)
         return ERROR_INT("b not made", procName, 1);
     *pvc = b;
 
@@ -790,7 +790,7 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     ptaGetPt(ptad, 3, &b[6], &b[7]);
 
     for (i = 0; i < 8; i++) {
-        if ((a[i] = (l_float32 *)CALLOC(8, sizeof(l_float32))) == NULL)
+        if ((a[i] = (l_float32 *)LEPT_CALLOC(8, sizeof(l_float32))) == NULL)
             return ERROR_INT("a[i] not made", procName, 1);
     }
 
@@ -830,7 +830,7 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     gaussjordan(a, b, 8);
 
     for (i = 0; i < 8; i++)
-        FREE(a[i]);
+        LEPT_FREE(a[i]);
 
     return 0;
 }
