@@ -540,13 +540,12 @@ BOX  *box;
     if (connectivity == 4) {
         if ((box = pixSeedfill4BB(pixs, stack, x, y)) == NULL)
             return (BOX *)ERROR_PTR("box not made", procName, NULL);
-    }
-    else if (connectivity == 8) {
+    } else if (connectivity == 8) {
         if ((box = pixSeedfill8BB(pixs, stack, x, y)) == NULL)
             return (BOX *)ERROR_PTR("box not made", procName, NULL);
-    }
-    else
+    } else {
         return (BOX *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
+    }
 
     return box;
 }
@@ -1055,7 +1054,7 @@ L_STACK  *auxstack;
     PROCNAME("pushFillsegBB");
 
     if (!stack) {
-        L_ERROR(procName, "stack not defined");
+        L_ERROR("stack not defined\n", procName);
         return;
     }
 
@@ -1066,16 +1065,16 @@ L_STACK  *auxstack;
 
     if (y + dy >= 0 && y + dy <= ymax) {
         if ((auxstack = stack->auxstack) == NULL) {
-            L_ERROR("auxstack not defined", procName);
+            L_ERROR("auxstack not defined\n", procName);
             return;
         }
 
             /* Get a fillseg to use */
-        if (lstackGetCount(auxstack) > 0)
+        if (lstackGetCount(auxstack) > 0) {
             fseg = (FILLSEG *)lstackRemove(auxstack);
-        else {
-            if ((fseg = (FILLSEG *)CALLOC(1, sizeof(FILLSEG))) == NULL) {
-                L_ERROR("fillseg not made", procName);
+        } else {
+            if ((fseg = (FILLSEG *)LEPT_CALLOC(1, sizeof(FILLSEG))) == NULL) {
+                L_ERROR("fillseg not made\n", procName);
                 return;
             }
         }
@@ -1120,22 +1119,22 @@ L_STACK  *auxstack;
     PROCNAME("pushFillseg");
 
     if (!stack) {
-        L_ERROR(procName, "stack not defined");
+        L_ERROR("stack not defined\n", procName);
         return;
     }
 
     if (y + dy >= 0 && y + dy <= ymax) {
         if ((auxstack = stack->auxstack) == NULL) {
-            L_ERROR("auxstack not defined", procName);
+            L_ERROR("auxstack not defined\n", procName);
             return;
         }
 
             /* Get a fillseg to use */
-        if (lstackGetCount(auxstack) > 0)
+        if (lstackGetCount(auxstack) > 0) {
             fseg = (FILLSEG *)lstackRemove(auxstack);
-        else {
-            if ((fseg = (FILLSEG *)CALLOC(1, sizeof(FILLSEG))) == NULL) {
-                L_ERROR("fillseg not made", procName);
+        } else {
+            if ((fseg = (FILLSEG *)LEPT_CALLOC(1, sizeof(FILLSEG))) == NULL) {
+                L_ERROR("fillseg not made\n", procName);
                 return;
             }
         }
@@ -1178,11 +1177,11 @@ L_STACK  *auxstack;
     PROCNAME("popFillseg");
 
     if (!stack) {
-        L_ERROR("stack not defined", procName);
+        L_ERROR("stack not defined\n", procName);
         return;
     }
     if ((auxstack = stack->auxstack) == NULL) {
-        L_ERROR("auxstack not defined", procName);
+        L_ERROR("auxstack not defined\n", procName);
         return;
     }
 
