@@ -142,9 +142,9 @@ l_uint32  *row1, *row2;
         return ERROR_INT("&score not defined", procName, 1);
     *pscore = 0.0;
     if (!pix1 || pixGetDepth(pix1) != 1)
-        return ERROR_INT("pix1 not 1 bpp", procName, 1);
+        return ERROR_INT("pix1 undefined or not 1 bpp", procName, 1);
     if (!pix2 || pixGetDepth(pix2) != 1)
-        return ERROR_INT("pix2 not 1 bpp", procName, 1);
+        return ERROR_INT("pix2 undefined or not 1 bpp", procName, 1);
     if (!tab)
         return ERROR_INT("tab not defined", procName, 1);
     if (area1 <= 0 || area2 <= 0)
@@ -437,9 +437,9 @@ l_int32    threshold;
     PROCNAME("pixCorrelationScoreThresholded");
 
     if (!pix1 || pixGetDepth(pix1) != 1)
-        return ERROR_INT("pix1 not 1 bpp", procName, 0);
+        return ERROR_INT("pix1 undefined or not 1 bpp", procName, 0);
     if (!pix2 || pixGetDepth(pix2) != 1)
-        return ERROR_INT("pix2 not 1 bpp", procName, 0);
+        return ERROR_INT("pix2 undefined or not 1 bpp", procName, 0);
     if (!tab)
         return ERROR_INT("tab not defined", procName, 0);
     if (area1 <= 0 || area2 <= 0)
@@ -531,7 +531,7 @@ l_int32    threshold;
                 }
 
                     /* If the count is over the threshold, no need to
-                     * calculate any futher.  Likewise, return early if the
+                     * calculate any further.  Likewise, return early if the
                      * count plus the maximum count attainable from further
                      * rows is below the threshold. */
                 if (count >= threshold) return TRUE;
@@ -741,9 +741,9 @@ PIX     *pixt;
         return ERROR_INT("&score not defined", procName, 1);
     *pscore = 0.0;
     if (!pix1 || pixGetDepth(pix1) != 1)
-        return ERROR_INT("pix1 not 1 bpp", procName, 1);
+        return ERROR_INT("pix1 undefined or not 1 bpp", procName, 1);
     if (!pix2 || pixGetDepth(pix2) != 1)
-        return ERROR_INT("pix2 not 1 bpp", procName, 1);
+        return ERROR_INT("pix2 undefined or not 1 bpp", procName, 1);
     if (!tab)
         return ERROR_INT("tab not defined", procName, 1);
     if (!area1 || !area2)
@@ -840,9 +840,9 @@ PIX     *pixt;
         return ERROR_INT("&score not defined", procName, 1);
     *pscore = 0.0;
     if (!pix1 || pixGetDepth(pix1) != 1)
-        return ERROR_INT("pix1 not 1 bpp", procName, 1);
+        return ERROR_INT("pix1 undefined or not 1 bpp", procName, 1);
     if (!pix2 || pixGetDepth(pix2) != 1)
-        return ERROR_INT("pix2 not 1 bpp", procName, 1);
+        return ERROR_INT("pix2 undefined or not 1 bpp", procName, 1);
     if (!tab)
         return ERROR_INT("tab not defined", procName, 1);
     if (!area1 || !area2)
