@@ -33,6 +33,8 @@
  *
  *      Top-level single page dewarper
  *          l_int32            dewarpSinglePage()
+ *          l_int32            dewarpSinglePageInit()
+ *          l_int32            dewarpSinglePageRun()
  *
  *      Operations on dewarpa
  *          l_int32            dewarpaListPages()
@@ -56,9 +58,13 @@
 static l_int32 dewarpaTestForValidModel(L_DEWARPA *dewa, L_DEWARP *dew,
                                         l_int32 notests);
 
+#ifndef  NO_CONSOLE_IO
+#define  DEBUG_INVALID_MODELS      0   /* set this to 1 for debuging */
+#endif  /* !NO_CONSOLE_IO */
+
     /* Special parameter values */
 static const l_int32     GRAYIN_VALUE = 200;
-
+ 
 
 /*----------------------------------------------------------------------*
  *                   Top-level single page dewarper                     *
@@ -90,11 +96,8 @@ dewarpSinglePage(PIX         *pixs,
                  L_DEWARPA  **pdewa,
                  l_int32      debug)
 {
-const char  *debugfile;
-l_int32      vsuccess, ret;
-L_DEWARP    *dew;
-L_DEWARPA   *dewa;
-PIX         *pix1, *pixb;
+L_DEWARPA  *dewa;
+PIX        *pixb;
 
     PROCNAME("dewarpSinglePage");
 
@@ -105,41 +108,14 @@ PIX         *pix1, *pixb;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
 
-    dewa = dewarpaCreate(1, 0, 1, 0, -1);
-    dewarpaUseBothArrays(dewa, use_both);
-
-         /* Generate a binary image, if necessary */
-    if (pixGetDepth(pixs) > 1) {
-        pix1 = pixConvertTo8(pixs, 0);
-        if (adaptive)
-            pixb = pixAdaptThresholdToBinary(pix1, NULL, 1.0);
-        else
-            pixb = pixThresholdToBinary(pix1, thresh);
-        pixDestroy(&pix1);
-    } else {
-        pixb = pixClone(pixs);
-    }
-
-        /* Generate the page model */
-    lept_mkdir("lept");
-    dew = dewarpCreate(pixb, 0);
-    dewarpaInsertDewarp(dewa, dew);
-    debugfile = (debug) ? "/tmp/lept/singlepage_model.pdf" : NULL;
-    dewarpBuildPageModel(dew, debugfile);
-    dewarpaModelStatus(dewa, 0, &vsuccess, NULL);
-    if (vsuccess == 0) {
-        L_ERROR("failure to build model\n", procName);
-        pixDestroy(&pixb);
+    dewarpSinglePageInit(pixs, thresh, adaptive, use_both, &pixb, &dewa);
+    if (!pixb) {
         dewarpaDestroy(&dewa);
-        *ppixd = pixCopy(NULL, pixs);
-        return 0;
+        return ERROR_INT("pixb not made", procName, 1);
     }
 
-        /* Apply the page model */
-    debugfile = (debug) ? "/tmp/lept/singlepage_apply.pdf" : NULL;
-    ret = dewarpaApplyDisparity(dewa, 0, pixs, 255, 0, 0, ppixd, debugfile);
-    if (ret)
-        L_ERROR("invalid model; failure to apply disparity\n", procName);
+    dewarpSinglePageRun(pixs, pixb, dewa, ppixd, debug);
+
     if (pdewa)
         *pdewa = dewa;
     else
@@ -149,6 +125,127 @@ PIX         *pix1, *pixb;
 }
 
 
+/*!
+ *  dewarpSinglePageInit()
+ *
+ *      Input:  pixs (with text, any depth)
+ *              thresh (for global thresholding to 1 bpp; ignored otherwise)
+ *              adaptive (1 for adaptive thresholding; 0 for global threshold)
+ *              use_both (1 for horizontal and vertical; 0 for vertical only)
+ *              &pixb (<return> 1 bpp image)
+ *              &dewa (<return> initialized dewa)
+ *      Return: 0 if OK, 1 on error (list of page numbers), or null on error
+ *
+ *  Notes:
+ *      (1) This binarizes the input pixs if necessary, returning the
+ *          binarized image.  It also initializes the dewa to default values
+ *          for the model parameters.
+ *      (2) If pixs is 1 bpp, the parameters @adaptive and @thresh are ignored.
+ *      (3) To change the model parameters, call dewarpaSetCurvatures()
+ *          before running dewarpSinglePageRun().  For example:
+ *             dewarpSinglePageInit(pixs, 0, 1, 1, &pixb, &dewa);
+ *             dewarpaSetCurvatures(dewa, 250, -1, -1, 80, 70, 150);
+ *             dewarpSinglePageRun(pixs, pixb, dewa, &pixd, 0);
+ *             dewarpaDestroy(&dewa);
+ *             pixDestroy(&pixb);
+ */
+l_int32
+dewarpSinglePageInit(PIX         *pixs,
+                     l_int32      thresh,
+                     l_int32      adaptive,
+                     l_int32      use_both,
+                     PIX        **ppixb,
+                     L_DEWARPA  **pdewa)
+{
+PIX  *pix1;
+
+    PROCNAME("dewarpSinglePageInit");
+
+    if (ppixb) *ppixb = NULL;
+    if (pdewa) *pdewa = NULL;
+    if (!ppixb || !pdewa)
+        return ERROR_INT("&pixb and &dewa not both defined", procName, 1);
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+
+    *pdewa = dewarpaCreate(1, 0, 1, 0, -1);
+    dewarpaUseBothArrays(*pdewa, use_both);
+
+         /* Generate a binary image, if necessary */
+    if (pixGetDepth(pixs) > 1) {
+        pix1 = pixConvertTo8(pixs, 0);
+        if (adaptive)
+            *ppixb = pixAdaptThresholdToBinary(pix1, NULL, 1.0);
+        else
+            *ppixb = pixThresholdToBinary(pix1, thresh);
+        pixDestroy(&pix1);
+    } else {
+        *ppixb = pixClone(pixs);
+    }
+    return 0;
+}
+
+
+/*!
+ *  dewarpSinglePageRun()
+ *
+ *      Input:  pixs (any depth)
+ *              pixb (1 bpp)
+ *              dewa (initialized)
+ *              &pixd (<return> dewarped result)
+ *              debug (1 for debugging output, 0 otherwise)
+ *      Return: 0 if OK, 1 on error (list of page numbers), or null on error
+ *
+ *  Notes:
+ *      (1) Dewarps pixs and returns the result in &pixd.
+ *      (2) The model parameters must be set before calling this.
+ *      (3) If a model cannot be built, this returns a copy of pixs in &pixd.
+ */
+l_int32
+dewarpSinglePageRun(PIX        *pixs,
+                    PIX        *pixb,
+                    L_DEWARPA  *dewa,
+                    PIX       **ppixd,
+                    l_int32     debug)
+{
+const char  *debugfile;
+l_int32      vsuccess, ret;
+L_DEWARP    *dew;
+
+    PROCNAME("dewarpSinglePageRun");
+
+    if (!ppixd)
+        return ERROR_INT("&pixd not defined", procName, 1);
+    *ppixd = NULL;
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+    if (!pixb)
+        return ERROR_INT("pixs not defined", procName, 1);
+    if (!dewa)
+        return ERROR_INT("dewa not defined", procName, 1);
+
+        /* Generate the page model */
+    lept_mkdir("lept/dewarp");
+    dew = dewarpCreate(pixb, 0);
+    dewarpaInsertDewarp(dewa, dew);
+    debugfile = (debug) ? "/tmp/lept/dewarp/singlepage_model.pdf" : NULL;
+    dewarpBuildPageModel(dew, debugfile);
+    dewarpaModelStatus(dewa, 0, &vsuccess, NULL);
+    if (vsuccess == 0) {
+        L_ERROR("failure to build model for vertical disparity\n", procName);
+        *ppixd = pixCopy(NULL, pixs);
+        return 0;
+    }
+
+        /* Apply the page model */
+    debugfile = (debug) ? "/tmp/lept/dewarp/singlepage_apply.pdf" : NULL;
+    ret = dewarpaApplyDisparity(dewa, 0, pixs, 255, 0, 0, ppixd, debugfile);
+    if (ret)
+        L_ERROR("invalid model; failure to apply disparity\n", procName);
+    return 0;
+}
+
+
 /*----------------------------------------------------------------------*
  *                        Operations on dewarpa                         *
  *----------------------------------------------------------------------*/
@@ -575,6 +672,16 @@ L_DEWARP  *dew;
         fprintf(stderr, "  nlines = %d\n", dew->nlines);
         fprintf(stderr, "  w = %d, h = %d, nx = %d, ny = %d\n",
                 dew->w, dew->h, dew->nx, dew->ny);
+        if (dew->sampvdispar)
+            fprintf(stderr, "  Vertical disparity builds:\n"
+                    "    (min,max,abs-diff) line curvature = (%d,%d,%d)\n",
+                    dew->mincurv, dew->maxcurv, dew->maxcurv - dew->mincurv);
+        if (dew->samphdispar)
+            fprintf(stderr, "  Horizontal disparity builds:\n"
+                    "    left edge slope = %d, right edge slope = %d\n"
+                    "    (left,right,abs-diff) edge curvature = (%d,%d,%d)\n",
+                    dew->leftslope, dew->rightslope, dew->leftcurv,
+                    dew->rightcurv, L_ABS(dew->leftcurv - dew->rightcurv));
     }
     return 0;
 }
@@ -713,7 +820,13 @@ l_int32  maxcurv, diffcurv, diffedge;
         diffcurv <= dewa->max_diff_linecurv) {
         dew->vvalid = 1;
     } else {
-        L_INFO("invalid vert model for page %d\n", procName, dew->pageno);
+        L_INFO("invalid vert model for page %d:\n", procName, dew->pageno);
+#if DEBUG_INVALID_MODELS
+        fprintf(stderr, "  max line curv = %d, max allowed = %d\n",
+                maxcurv, dewa->max_linecurv);
+        fprintf(stderr, "  diff line curv = %d, max allowed = %d\n",
+                diffcurv, dewa->max_diff_linecurv);
+#endif  /* DEBUG_INVALID_MODELS */
     }
 
         /* If a horizontal (edge) model exists, test for validity. */
@@ -726,7 +839,19 @@ l_int32  maxcurv, diffcurv, diffedge;
             diffedge <= dewa->max_diff_edgecurv) {
             dew->hvalid = 1;
         } else {
-            L_INFO("invalid horiz model for page %d\n", procName, dew->pageno);
+            L_INFO("invalid horiz model for page %d:\n", procName, dew->pageno);
+#if DEBUG_INVALID_MODELS
+            fprintf(stderr, "  left edge slope = %d, max allowed = %d\n",
+                    dew->leftslope, dewa->max_edgeslope);
+            fprintf(stderr, "  right edge slope = %d, max allowed = %d\n",
+                    dew->rightslope, dewa->max_edgeslope);
+            fprintf(stderr, "  left edge curv = %d, max allowed = %d\n",
+                    dew->leftcurv, dewa->max_edgecurv);
+            fprintf(stderr, "  right edge curv = %d, max allowed = %d\n",
+                    dew->rightcurv, dewa->max_edgecurv);
+            fprintf(stderr, "  diff edge curv = %d, max allowed = %d\n",
+                    diffedge, dewa->max_diff_edgecurv);
+#endif  /* DEBUG_INVALID_MODELS */
         }
     }
 
@@ -741,7 +866,6 @@ l_int32  maxcurv, diffcurv, diffedge;
  *              scalefact (on contour images; typ. 0.5)
  *              first (first page model to render)
  *              last (last page model to render; use 0 to go to end)
- *              fontdir (for text bitmap fonts)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -752,8 +876,7 @@ l_int32
 dewarpaShowArrays(L_DEWARPA   *dewa,
                   l_float32    scalefact,
                   l_int32      first,
-                  l_int32      last,
-                  const char  *fontdir)
+                  l_int32      last)
 {
 char       buf[256];
 char      *pathname;
@@ -773,10 +896,10 @@ PIXA      *pixa;
     if (last < first)
         return ERROR_INT("last < first", procName, 1);
 
-    lept_rmdir("lept");
-    lept_mkdir("lept");
-    if ((bmf = bmfCreate(fontdir, 8)) == NULL)
-              L_ERROR("bmf not made; page info not displayed", procName);
+    lept_rmdir("lept/dewarp1");  /* temp directory for contour plots */
+    lept_mkdir("lept/dewarp1");
+    if ((bmf = bmfCreate(NULL, 8)) == NULL)
+        L_ERROR("bmf not made; page info not displayed", procName);
 
     fprintf(stderr, "Generating contour plots\n");
     for (i = first; i <= last; i++) {
@@ -815,18 +938,18 @@ PIXA      *pixa;
         pixd = pixAddSingleTextblock(pixt, bmf, buf, 0x0000ff00,
                                      L_ADD_BELOW, NULL);
         snprintf(buf, sizeof(buf), "arrays_%04d.png", i);
-        pathname = genPathname("/tmp/lept", buf);
+        pathname = genPathname("/tmp/lept/dewarp1", buf);
         pixWrite(pathname, pixd, IFF_PNG);
         pixaDestroy(&pixa);
         pixDestroy(&pixt);
         pixDestroy(&pixd);
-        FREE(pathname);
+        LEPT_FREE(pathname);
     }
     bmfDestroy(&bmf);
     fprintf(stderr, "\n");
 
     fprintf(stderr, "Generating pdf of contour plots\n");
-    convertFilesToPdf("/tmp/lept", "arrays_", 90, 1.0, L_FLATE_ENCODE,
+    convertFilesToPdf("/tmp/lept/dewarp1", "arrays_", 90, 1.0, L_FLATE_ENCODE,
                       0, "Disparity arrays", "/tmp/lept/disparity_arrays.pdf");
     fprintf(stderr, "Output written to: /tmp/lept/disparity_arrays.pdf\n");
     return 0;
@@ -837,22 +960,22 @@ PIXA      *pixa;
  *  dewarpDebug()
  *
  *      Input:  dew
- *              subdir (a subdirectory of /tmp; e.g., "dew1")
+ *              subdirs (one or more subdirectories of /tmp; e.g., "dew1")
  *              index (to help label output images; e.g., the page number)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
  *      (1) Prints dewarp fields and generates disparity array contour images.
  *          The contour images are written to file:
- *                /tmp/[subdir]/pixv_[index].png
+ *                /tmp/[subdirs]/pixv_[index].png
  */
 l_int32
 dewarpDebug(L_DEWARP    *dew,
-            const char  *subdir,
+            const char  *subdirs,
             l_int32      index)
 {
-char     outdir[256], fname[64];
-char    *pathname;
+char     fname[64];
+char    *outdir, *pathname;
 l_int32  svd, shd;
 PIX     *pixv, *pixh;
 
@@ -860,8 +983,8 @@ PIX     *pixv, *pixh;
 
     if (!dew)
         return ERROR_INT("dew not defined", procName, 1);
-    if (!subdir)
-        return ERROR_INT("subdir not defined", procName, 1);
+    if (!subdirs)
+        return ERROR_INT("subdirs not defined", procName, 1);
 
     fprintf(stderr, "pageno = %d, hasref = %d, refpage = %d\n",
             dew->pageno, dew->hasref, dew->refpage);
@@ -880,6 +1003,8 @@ PIX     *pixv, *pixh;
                     dew->mincurv, dew->maxcurv, dew->maxcurv - dew->mincurv);
         }
         if (shd) {
+            fprintf(stderr, "(left edge slope = %d, right edge slope = %d\n",
+                    dew->leftslope, dew->rightslope);
             fprintf(stderr, "(left,right,abs-diff) edge curvature = "
                     "(%d,%d,%d)\n", dew->leftcurv, dew->rightcurv,
                     L_ABS(dew->leftcurv - dew->rightcurv));
@@ -891,15 +1016,15 @@ PIX     *pixv, *pixh;
     }
 
     dewarpPopulateFullRes(dew, NULL, 0, 0);
-    lept_mkdir(subdir);
-    snprintf(outdir, sizeof(outdir), "/tmp/%s", subdir);
+    lept_mkdir(subdirs);
+    outdir = pathJoin("/tmp", subdirs);
     if (svd) {
         pixv = fpixRenderContours(dew->fullvdispar, 3.0, 0.15);
         snprintf(fname, sizeof(fname), "pixv_%d.png", index);
         pathname = genPathname(outdir, fname);
         pixWrite(pathname, pixv, IFF_PNG);
         pixDestroy(&pixv);
-        FREE(pathname);
+        LEPT_FREE(pathname);
     }
     if (shd) {
         pixh = fpixRenderContours(dew->fullhdispar, 3.0, 0.15);
@@ -907,8 +1032,9 @@ PIX     *pixv, *pixh;
         pathname = genPathname(outdir, fname);
         pixWrite(pathname, pixh, IFF_PNG);
         pixDestroy(&pixh);
-        FREE(pathname);
+        LEPT_FREE(pathname);
     }
+    LEPT_FREE(outdir);
     return 0;
 }
 
@@ -920,7 +1046,6 @@ PIX     *pixv, *pixh;
  *              sarray (of indexed input images)
  *              boxa (crop boxes for input images; can be null)
  *              firstpage, lastpage
- *              fontdir (for text bitmap fonts)
  *              pdfout (filename)
  *      Return: 0 if OK, 1 on error
  *
@@ -938,7 +1063,6 @@ dewarpShowResults(L_DEWARPA   *dewa,
                   BOXA        *boxa,
                   l_int32      firstpage,
                   l_int32      lastpage,
-                  const char  *fontdir,
                   const char  *pdfout)
 {
 char       bufstr[256];
@@ -961,10 +1085,9 @@ PIXA      *pixa;
     if (firstpage > lastpage)
         return ERROR_INT("invalid first/last page numbers", procName, 1);
 
-    lept_rmdir("dewarp_pdfout");
-    lept_mkdir("dewarp_pdfout");
-    if ((bmf = bmfCreate(fontdir, 6)) == NULL)
-        L_ERROR("bmf not made; page info not displayed", procName);
+    lept_rmdir("lept/dewarp_pdfout");
+    lept_mkdir("lept/dewarp_pdfout");
+    bmf = bmfCreate(NULL, 6);
 
     fprintf(stderr, "Dewarping and generating s/by/s view\n");
     for (i = firstpage; i <= lastpage; i++) {
@@ -998,7 +1121,7 @@ PIXA      *pixa;
             snprintf(bufstr, sizeof(bufstr), "Page %d; no dewarp\n", i);
         pixt2 = pixAddSingleTextblock(pixt1, bmf, bufstr, 0x0000ff00,
                                       L_ADD_BELOW, 0);
-        snprintf(bufstr, sizeof(bufstr), "/tmp/dewarp_pdfout/%05d", i);
+        snprintf(bufstr, sizeof(bufstr), "/tmp/lept/dewarp_pdfout/%05d", i);
         pixWrite(bufstr, pixt2, IFF_JFIF_JPEG);
         pixaDestroy(&pixa);
         pixDestroy(&pixs);
@@ -1008,11 +1131,11 @@ PIXA      *pixa;
     fprintf(stderr, "\n");
 
     fprintf(stderr, "Generating pdf of result\n");
-    convertFilesToPdf("/tmp/dewarp_pdfout", NULL, 100, 1.0, L_JPEG_ENCODE,
+    convertFilesToPdf("/tmp/lept/dewarp_pdfout", NULL, 100, 1.0, L_JPEG_ENCODE,
                       0, "Dewarp sequence", pdfout);
     outpath = genPathname(pdfout, NULL);
     fprintf(stderr, "Output written to: %s\n", outpath);
-    FREE(outpath);
+    LEPT_FREE(outpath);
     bmfDestroy(&bmf);
     return 0;
 }
