@@ -67,11 +67,13 @@
  *
  *      Dnaa creation, destruction
  *          L_DNAA      *l_dnaaCreate()
+ *          L_DNAA      *l_dnaaCreateFull()
+ *          l_int32      l_dnaaTruncate()
  *          void        *l_dnaaDestroy()
  *
  *      Add Dna to Dnaa
  *          l_int32      l_dnaaAddDna()
- *          l_int32      l_dnaaExtendArray()
+ *          static l_int32  l_dnaaExtendArray()
  *
  *      Dnaa accessors
  *          l_int32      l_dnaaGetCount()
@@ -88,6 +90,23 @@
  *          l_int32      l_dnaaWrite()
  *          l_int32      l_dnaaWriteStream()
  *
+ *      DnaHash creation, destruction
+ *          L_DNAHASH   *l_dnaHashCreate()
+ *          void        *l_dnaHashDestroy()
+ *
+ *      DnaHash: Accessors and modifiers                      *
+ *          l_int32     *l_dnaHashGetCount()
+ *          l_int32     *l_dnaHashGetTotalCount()
+ *          L_DNA       *l_dnaHashGetDna()
+ *          void        *l_dnaHashAdd()
+ *
+ *      DnaHash: Operations on Dna
+ *          L_DNAHASH   *l_dnaHashCreateFromDna()
+ *          l_int32      l_dnaRemoveDupsByHash()
+ *          l_int32      l_dnaMakeHistoByHash()
+ *          L_DNA       *l_dnaIntersectionByHash()
+ *          l_int32      l_dnaFindValByHash()
+ *
  *      Other Dna functions
  *          L_DNA       *l_dnaMakeDelta()
  *          NUMA        *l_dnaConvertToNuma()
@@ -96,8 +115,8 @@
  *
  *    (1) The Dna is a struct holding an array of doubles.  It can also
  *        be used to store l_int32 values, up to the full precision
- *        of int32.  Use it whenever integers larger than a few million
- *        need to be stored.
+ *        of int32.  Always use it whenever integers larger than a
+ *        few million need to be stored.
  *
  *    (2) Always use the accessors in this file, never the fields directly.
  *
@@ -114,7 +133,7 @@
  *
  *       * to obtain a value from the array, use either l_dnaGetIValue()
  *         or l_dnaGetDValue(), depending on whether you are retrieving
- *         an integer or a float.  This avoids doing an explicit cast,
+ *         an integer or a float64.  This avoids doing an explicit cast,
  *         such as
  *           (a) return a l_float64 and cast it to an l_int32
  *           (b) cast the return directly to (l_float64 *) to
@@ -177,9 +196,9 @@ L_DNA  *da;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((da = (L_DNA *)CALLOC(1, sizeof(L_DNA))) == NULL)
+    if ((da = (L_DNA *)LEPT_CALLOC(1, sizeof(L_DNA))) == NULL)
         return (L_DNA *)ERROR_PTR("da not made", procName, NULL);
-    if ((da->array = (l_float64 *)CALLOC(n, sizeof(l_float64))) == NULL)
+    if ((da->array = (l_float64 *)LEPT_CALLOC(n, sizeof(l_float64))) == NULL)
         return (L_DNA *)ERROR_PTR("double array not made", procName, NULL);
 
     da->nalloc = n;
@@ -259,7 +278,7 @@ L_DNA   *da;
 
     da = l_dnaCreate(size);
     if (copyflag == L_INSERT) {
-        if (da->array) FREE(da->array);
+        if (da->array) LEPT_FREE(da->array);
         da->array = darray;
         da->n = size;
     } else {  /* just copy the contents */
@@ -331,8 +350,8 @@ L_DNA  *da;
     l_dnaChangeRefcount(da, -1);
     if (l_dnaGetRefcount(da) <= 0) {
         if (da->array)
-            FREE(da->array);
-        FREE(da);
+            LEPT_FREE(da->array);
+        LEPT_FREE(da);
     }
 
     *pda = NULL;
@@ -345,6 +364,9 @@ L_DNA  *da;
  *
  *      Input:  da
  *      Return: copy of l_dna, or null on error
+ *
+ *  Notes:
+ *      (1) This removes unused ptrs above da->n.
  */
 L_DNA *
 l_dnaCopy(L_DNA  *da)
@@ -357,7 +379,7 @@ L_DNA   *dac;
     if (!da)
         return (L_DNA *)ERROR_PTR("da not defined", procName, NULL);
 
-    if ((dac = l_dnaCreate(da->nalloc)) == NULL)
+    if ((dac = l_dnaCreate(da->n)) == NULL)
         return (L_DNA *)ERROR_PTR("dac not made", procName, NULL);
     dac->startx = da->startx;
     dac->delx = da->delx;
@@ -770,7 +792,7 @@ l_int32  *array;
         return (l_int32 *)ERROR_PTR("da not defined", procName, NULL);
 
     n = l_dnaGetCount(da);
-    if ((array = (l_int32 *)CALLOC(n, sizeof(l_int32))) == NULL)
+    if ((array = (l_int32 *)LEPT_CALLOC(n, sizeof(l_int32))) == NULL)
         return (l_int32 *)ERROR_PTR("array not made", procName, NULL);
     for (i = 0; i < n; i++) {
         l_dnaGetIValue(da, i, &ival);
@@ -795,7 +817,7 @@ l_int32  *array;
  *          directly on the bare array of the l_dna.
  *      (2) Very important: for L_NOCOPY, any writes to the array
  *          will be in the l_dna.  Do not write beyond the size of
- *          the count field, because it will not be accessable
+ *          the count field, because it will not be accessible
  *          from the l_dna!  If necessary, be sure to set the count
  *          field to a larger number (such as the alloc size)
  *          BEFORE calling this function.  Creating with l_dnaMakeConstant()
@@ -817,7 +839,7 @@ l_float64  *array;
         array = da->array;
     } else {  /* copyflag == L_COPY */
         n = l_dnaGetCount(da);
-        if ((array = (l_float64 *)CALLOC(n, sizeof(l_float64))) == NULL)
+        if ((array = (l_float64 *)LEPT_CALLOC(n, sizeof(l_float64))) == NULL)
             return (l_float64 *)ERROR_PTR("array not made", procName, NULL);
         for (i = 0; i < n; i++)
             array[i] = da->array[i];
@@ -879,6 +901,10 @@ l_dnaGetParameters(L_DNA     *da,
 {
     PROCNAME("l_dnaGetParameters");
 
+    if (pstartx) *pstartx = 0.0;
+    if (pdelx) *pdelx = 1.0;
+    if (!pstartx && !pdelx)
+        return ERROR_INT("neither &startx nor &delx are defined", procName, 1);
     if (!da)
         return ERROR_INT("da not defined", procName, 1);
 
@@ -976,6 +1002,9 @@ L_DNA  *da;
  *
  *      Input:  stream
  *      Return: da, or null on error
+ *
+ *  Notes:
+ *      (1) fscanf takes %lf to read a double; fprintf takes %f to write it.
  */
 L_DNA *
 l_dnaReadStream(FILE  *fp)
@@ -1067,13 +1096,13 @@ l_float64  startx, delx;
     fprintf(fp, "\nL_Dna Version %d\n", DNA_VERSION_NUMBER);
     fprintf(fp, "Number of numbers = %d\n", n);
     for (i = 0; i < n; i++)
-        fprintf(fp, "  [%d] = %lf\n", i, da->array[i]);
+        fprintf(fp, "  [%d] = %f\n", i, da->array[i]);
     fprintf(fp, "\n");
 
         /* Optional data */
     l_dnaGetParameters(da, &startx, &delx);
     if (startx != 0.0 || delx != 1.0)
-        fprintf(fp, "startx = %lf, delx = %lf\n", startx, delx);
+        fprintf(fp, "startx = %f, delx = %f\n", startx, delx);
 
     return 0;
 }
@@ -1099,9 +1128,9 @@ L_DNAA  *daa;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((daa = (L_DNAA *)CALLOC(1, sizeof(L_DNAA))) == NULL)
+    if ((daa = (L_DNAA *)LEPT_CALLOC(1, sizeof(L_DNAA))) == NULL)
         return (L_DNAA *)ERROR_PTR("daa not made", procName, NULL);
-    if ((daa->dna = (L_DNA **)CALLOC(n, sizeof(L_DNA *))) == NULL)
+    if ((daa->dna = (L_DNA **)LEPT_CALLOC(n, sizeof(L_DNA *))) == NULL)
         return (L_DNAA *)ERROR_PTR("l_dna ptr array not made", procName, NULL);
 
     daa->nalloc = n;
@@ -1111,6 +1140,76 @@ L_DNAA  *daa;
 }
 
 
+/*!
+ *  l_dnaaCreateFull()
+ *
+ *      Input:  nptr: size of dna ptr array to be alloc'd
+ *              n: size of individual dna arrays to be alloc'd (0 for default)
+ *      Return: daa, or null on error
+ *
+ *  Notes:
+ *      (1) This allocates a dnaa and fills the array with allocated dnas.
+ *          In use, after calling this function, use
+ *              l_dnaaAddNumber(dnaa, index, val);
+ *          to add val to the index-th dna in dnaa.
+ */
+L_DNAA *
+l_dnaaCreateFull(l_int32  nptr,
+                 l_int32  n)
+{
+l_int32  i;
+L_DNAA  *daa;
+L_DNA   *da;
+
+    daa = l_dnaaCreate(nptr);
+    for (i = 0; i < nptr; i++) {
+        da = l_dnaCreate(n);
+        l_dnaaAddDna(daa, da, L_INSERT);
+    }
+
+    return daa;
+}
+
+
+/*!
+ *  l_dnaaTruncate()
+ *
+ *      Input:  daa
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This identifies the largest index containing a dna that
+ *          has any numbers within it, destroys all dna beyond that
+ *          index, and resets the count.
+ */
+l_int32
+l_dnaaTruncate(L_DNAA  *daa)
+{
+l_int32  i, n, nn;
+L_DNA   *da;
+
+    PROCNAME("l_dnaaTruncate");
+
+    if (!daa)
+        return ERROR_INT("daa not defined", procName, 1);
+
+    n = l_dnaaGetCount(daa);
+    for (i = n - 1; i >= 0; i--) {
+        da = l_dnaaGetDna(daa, i, L_CLONE);
+        if (!da)
+            continue;
+        nn = l_dnaGetCount(da);
+        l_dnaDestroy(&da);  /* the clone */
+        if (nn == 0)
+            l_dnaDestroy(&daa->dna[i]);
+        else
+            break;
+    }
+    daa->n = i + 1;
+    return 0;
+}
+
+
 /*!
  *  l_dnaaDestroy()
  *
@@ -1135,8 +1234,8 @@ L_DNAA  *daa;
 
     for (i = 0; i < daa->n; i++)
         l_dnaDestroy(&daa->dna[i]);
-    FREE(daa->dna);
-    FREE(daa);
+    LEPT_FREE(daa->dna);
+    LEPT_FREE(daa);
     *pdaa = NULL;
 
     return;
@@ -1156,8 +1255,8 @@ L_DNAA  *daa;
  */
 l_int32
 l_dnaaAddDna(L_DNAA  *daa,
-               L_DNA   *da,
-               l_int32  copyflag)
+             L_DNA   *da,
+             l_int32  copyflag)
 {
 l_int32  n;
 L_DNA   *dac;
@@ -1555,6 +1654,482 @@ L_DNA   *da;
 }
 
 
+/*--------------------------------------------------------------------------*
+ *                     Dna hash: Creation and destruction                   *
+ *--------------------------------------------------------------------------*/
+/*!
+ *  l_dnaHashCreate()
+ *
+ *      Input: nbuckets (the number of buckets in the hash table,
+ *                       which should be prime.)
+ *             initsize (initial size of each allocated numa; 0 for default)
+ *      Return: ptr to new dnahash, or null on error
+ *
+ *  Note: actual dna are created only as required by l_dnaHashAdd()
+ */
+L_DNAHASH *
+l_dnaHashCreate(l_int32  nbuckets,
+                l_int32  initsize)
+{
+L_DNAHASH  *dahash;
+
+    PROCNAME("l_dnaHashCreate");
+
+    if (nbuckets <= 0)
+        return (L_DNAHASH *)ERROR_PTR("negative hash size", procName, NULL);
+    if ((dahash = (L_DNAHASH *)LEPT_CALLOC(1, sizeof(L_DNAHASH))) == NULL)
+        return (L_DNAHASH *)ERROR_PTR("dahash not made", procName, NULL);
+    if ((dahash->dna = (L_DNA **)LEPT_CALLOC(nbuckets, sizeof(L_DNA *)))
+        == NULL) {
+        LEPT_FREE(dahash);
+        return (L_DNAHASH *)ERROR_PTR("dna ptr array not made", procName, NULL);
+    }
+
+    dahash->nbuckets = nbuckets;
+    dahash->initsize = initsize;
+    return dahash;
+}
+
+
+/*!
+ *  l_dnaHashDestroy()
+ *
+ *      Input:  &dahash (<to be nulled, if it exists>)
+ *      Return: void
+ */
+void
+l_dnaHashDestroy(L_DNAHASH **pdahash)
+{
+L_DNAHASH  *dahash;
+l_int32    i;
+
+    PROCNAME("l_dnaHashDestroy");
+
+    if (pdahash == NULL) {
+        L_WARNING("ptr address is NULL!\n", procName);
+        return;
+    }
+
+    if ((dahash = *pdahash) == NULL)
+        return;
+
+    for (i = 0; i < dahash->nbuckets; i++)
+        l_dnaDestroy(&dahash->dna[i]);
+    LEPT_FREE(dahash->dna);
+    LEPT_FREE(dahash);
+    *pdahash = NULL;
+}
+
+
+/*--------------------------------------------------------------------------*
+ *                   Dna hash: Accessors and modifiers                      *
+ *--------------------------------------------------------------------------*/
+/*!
+ *  l_dnaHashGetCount()
+ *
+ *      Input:  dahash
+ *      Return: nbuckets (allocated, or 0 on error)
+ */
+l_int32
+l_dnaHashGetCount(L_DNAHASH  *dahash)
+{
+l_int32  nbuckets;
+
+    PROCNAME("l_dnaHashGetCount");
+
+    if (!dahash)
+        return ERROR_INT("dahash not defined", procName, 0);
+    return dahash->nbuckets;
+}
+
+
+/*!
+ *  l_dnaHashGetTotalCount()
+ *
+ *      Input:  dahash
+ *      Return: n (number of numbers in all dna, or 0 on error)
+ */
+l_int32
+l_dnaHashGetTotalCount(L_DNAHASH  *dahash)
+{
+l_int32  nbuckets, i, n;
+L_DNA   *da;
+
+    PROCNAME("l_dnaHashGetTotalCount");
+
+    if (!dahash)
+        return ERROR_INT("dahash not defined", procName, 0);
+
+    for (i = 0, n = 0; i < dahash->nbuckets; i++) {
+        da = l_dnaHashGetDna(dahash, i, L_NOCOPY);
+        if (da)
+            n += l_dnaGetCount(da);
+    }
+
+    return n;
+}
+
+
+/*!
+ *  l_dnaHashGetDna()
+ *
+ *      Input:  dahash
+ *              key  (key to be hashed into a bucket number)
+ *              copyflag (L_NOCOPY, L_COPY, L_CLONE)
+ *      Return: ptr to numa
+ */
+L_DNA *
+l_dnaHashGetDna(L_DNAHASH  *dahash,
+                l_uint64    key,
+                l_int32     copyflag)
+{
+l_int32  bucket;
+L_DNA   *da;
+
+    PROCNAME("l_dnaHashGetDna");
+
+    if (!dahash)
+        return (L_DNA *)ERROR_PTR("dahash not defined", procName, NULL);
+    bucket = key % dahash->nbuckets;
+    da = dahash->dna[bucket];
+    if (da) {
+        if (copyflag == L_NOCOPY)
+            return da;
+        else if (copyflag == L_COPY)
+            return l_dnaCopy(da);
+        else
+            return l_dnaClone(da);
+    }
+    else
+        return NULL;
+}
+
+
+/*!
+ *  l_dnaHashAdd()
+ *
+ *      Input:  dahash
+ *              key  (key to be hashed into a bucket number)
+ *              value  (float value to be appended to the specific dna)
+ *      Return: 0 if OK; 1 on error
+ */
+l_int32
+l_dnaHashAdd(L_DNAHASH  *dahash,
+             l_uint64    key,
+             l_float64   value)
+{
+l_int32  bucket;
+L_DNA   *da;
+
+    PROCNAME("l_dnaHashAdd");
+
+    if (!dahash)
+        return ERROR_INT("dahash not defined", procName, 1);
+    bucket = key % dahash->nbuckets;
+    da = dahash->dna[bucket];
+    if (!da) {
+        if ((da = l_dnaCreate(dahash->initsize)) == NULL)
+            return ERROR_INT("da not made", procName, 1);
+        dahash->dna[bucket] = da;
+    }
+    l_dnaAddNumber(da, value);
+    return 0;
+}
+
+
+/*--------------------------------------------------------------------------*
+ *                      DnaHash: Operations on Dna                          *
+ *--------------------------------------------------------------------------*/
+/*!
+ *  l_dnaHashCreateFromDna()
+ *
+ *      Input:  da
+ *      Return: dahash if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) The values stored in the @dahash are indices into @da;
+ *          @dahash has no use without @da.
+ */
+L_DNAHASH *
+l_dnaHashCreateFromDna(L_DNA  *da)
+{
+l_int32     i, n;
+l_uint32    nsize;
+l_uint64    key;
+l_float64   val;
+L_DNAHASH  *dahash;
+
+    PROCNAME("l_dnaHashCreateFromDna");
+
+    if (!da)
+        return (L_DNAHASH *)ERROR_PTR("da not defined", procName, NULL);
+
+    n = l_dnaGetCount(da);
+    findNextLargerPrime(n / 20, &nsize);  /* buckets in hash table */
+
+    dahash = l_dnaHashCreate(nsize, 8);
+    for (i = 0; i < n; i++) {
+        l_dnaGetDValue(da, i, &val);
+        l_hashFloat64ToUint64(nsize, val, &key);
+        l_dnaHashAdd(dahash, key, (l_float64)i);
+    }
+
+    return dahash;
+}
+
+
+/*!
+ *  l_dnaRemoveDupsByHash()
+ *
+ *      Input:  das
+ *              &dad (<return> hash set)
+ *              &dahash (<optional return> dnahash used for lookup)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) Generates a dna with unique values.
+ *      (2) The dnahash is built up with dad to assure uniqueness.
+ *          It can be used to find if an element is in the set:
+ *              l_dnaFindValByHash(dad, dahash, val, &index)
+ */
+l_int32
+l_dnaRemoveDupsByHash(L_DNA       *das,
+                      L_DNA      **pdad,
+                      L_DNAHASH  **pdahash)
+{
+l_int32     i, n, index, items;
+l_uint32    nsize;
+l_uint64    key;
+l_float64   val;
+L_DNA      *dad;
+L_DNAHASH  *dahash;
+
+    PROCNAME("l_dnaRemoveDupsByHash");
+
+    if (pdahash) *pdahash = NULL;
+    if (!pdad)
+        return ERROR_INT("&dad not defined", procName, 1);
+    *pdad = NULL;
+    if (!das)
+        return ERROR_INT("das not defined", procName, 1);
+
+    n = l_dnaGetCount(das);
+    findNextLargerPrime(n / 20, &nsize);  /* buckets in hash table */
+    dahash = l_dnaHashCreate(nsize, 8);
+    dad = l_dnaCreate(n);
+    *pdad = dad;
+    for (i = 0, items = 0; i < n; i++) {
+        l_dnaGetDValue(das, i, &val);
+        l_dnaFindValByHash(dad, dahash, val, &index);
+        if (index < 0) {  /* not found */
+            l_hashFloat64ToUint64(nsize, val, &key);
+            l_dnaHashAdd(dahash, key, (l_float64)items);
+            l_dnaAddNumber(dad, val);
+            items++;
+        }
+    }
+
+    if (pdahash)
+        *pdahash = dahash;
+    else
+        l_dnaHashDestroy(&dahash);
+    return 0;
+}
+
+
+/*!
+ *  l_dnaMakeHistoByHash()
+ *
+ *      Input:  das
+ *              &dahash (<return> hash map: val --> index)
+ *              &dav (<return> array of values: index --> val)
+ *              &dac (<return> histo array of counts: index --> count)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) Generates and returns a dna of occurrences (histogram),
+ *          an aligned dna of values, and an associated hashmap.
+ *          The hashmap takes @dav and a value, and points into the
+ *          histogram in @dac.
+ *      (2) The dna of values, @dav, is aligned with the histogram @dac,
+ *          and is needed for fast lookup.  It is a hash set, because
+ *          the values are unique.
+ *      (3) Lookup is simple:
+ *              l_dnaFindValByHash(dav, dahash, val, &index);
+ *              if (index >= 0)
+ *                  l_dnaGetIValue(dac, index, &icount);
+ *              else
+ *                  icount = 0;
+ */
+l_int32
+l_dnaMakeHistoByHash(L_DNA       *das,
+                     L_DNAHASH  **pdahash,
+                     L_DNA      **pdav,
+                     L_DNA      **pdac)
+{
+l_int32     i, n, nitems, index, count;
+l_uint32    nsize;
+l_uint64    key;
+l_float64   val;
+L_DNA      *dac, *dav;
+L_DNAHASH  *dahash;
+
+    PROCNAME("l_dnaMakeHistoByHash");
+
+    if (pdahash) *pdahash = NULL;
+    if (pdac) *pdac = NULL;
+    if (pdav) *pdav = NULL;
+    if (!pdahash || !pdac || !pdav)
+        return ERROR_INT("&dahash, &dac, &dav not all defined", procName, 1);
+    if (!das)
+        return ERROR_INT("das not defined", procName, 1);
+    if ((n = l_dnaGetCount(das)) == 0)
+        return ERROR_INT("no data in das", procName, 1);
+
+    findNextLargerPrime(n / 20, &nsize);  /* buckets in hash table */
+    dahash = l_dnaHashCreate(nsize, 8);
+    dac = l_dnaCreate(n);  /* histogram */
+    dav = l_dnaCreate(n);  /* the values */
+    for (i = 0, nitems = 0; i < n; i++) {
+        l_dnaGetDValue(das, i, &val);
+            /* Is this value already stored in dav? */
+        l_dnaFindValByHash(dav, dahash, val, &index);
+        if (index >= 0) {  /* found */
+            l_dnaGetIValue(dac, (l_float64)index, &count);
+            l_dnaSetValue(dac, (l_float64)index, count + 1);
+        } else {  /* not found */
+            l_hashFloat64ToUint64(nsize, val, &key);
+            l_dnaHashAdd(dahash, key, (l_float64)nitems);
+            l_dnaAddNumber(dav, val);
+            l_dnaAddNumber(dac, 1);
+            nitems++;
+        }
+    }
+
+    *pdahash = dahash;
+    *pdac = dac;
+    *pdav = dav;
+    return 0;
+}
+
+
+/*!
+ *  l_dnaIntersectionByHash()
+ *
+ *      Input:  da1, da2
+ *      Return: dad (intersection of the number arrays), or null on error
+ *
+ *  Notes:
+ *      (1) This uses the same method for building the intersection set
+ *          as ptaIntersectionByHash() and sarrayIntersectionByHash().
+ */
+L_DNA *
+l_dnaIntersectionByHash(L_DNA  *da1,
+                        L_DNA  *da2)
+{
+l_int32     n1, n2, nsmall, nbuckets, i, index1, index2;
+l_uint32    nsize2;
+l_uint64    key;
+l_float64   val;
+L_DNAHASH  *dahash1, *dahash2;
+L_DNA      *da_small, *da_big, *dad;
+
+    PROCNAME("l_dnaIntersectionByHash");
+
+    if (!da1)
+        return (L_DNA *)ERROR_PTR("da1 not defined", procName, NULL);
+    if (!da2)
+        return (L_DNA *)ERROR_PTR("da2 not defined", procName, NULL);
+
+        /* Put the elements of the biggest array into a dnahash */
+    n1 = l_dnaGetCount(da1);
+    n2 = l_dnaGetCount(da2);
+    da_small = (n1 < n2) ? da1 : da2;   /* do not destroy da_small */
+    da_big = (n1 < n2) ? da2 : da1;   /* do not destroy da_big */
+    dahash1 = l_dnaHashCreateFromDna(da_big);
+
+        /* Build up the intersection of numbers.  Add to @dad
+         * if the number is in da_big (using dahash1) but hasn't
+         * yet been seen in the traversal of da_small (using dahash2). */
+    dad = l_dnaCreate(0);
+    nsmall = l_dnaGetCount(da_small);
+    findNextLargerPrime(nsmall / 20, &nsize2);  /* buckets in hash table */
+    dahash2 = l_dnaHashCreate(nsize2, 0);
+    nbuckets = l_dnaHashGetCount(dahash2);
+    for (i = 0; i < nsmall; i++) {
+        l_dnaGetDValue(da_small, i, &val);
+        l_dnaFindValByHash(da_big, dahash1, val, &index1);
+        if (index1 >= 0) {  /* found */
+            l_dnaFindValByHash(da_small, dahash2, val, &index2);
+            if (index2 == -1) {  /* not found */
+                l_dnaAddNumber(dad, val);
+                l_hashFloat64ToUint64(nbuckets, val, &key);
+                l_dnaHashAdd(dahash2, key, (l_float64)i);
+            }
+        }
+    }
+
+    l_dnaHashDestroy(&dahash1);
+    l_dnaHashDestroy(&dahash2);
+    return dad;
+}
+
+
+/*!
+ *  l_dnaFindValByHash()
+ *
+ *      Input:  da
+ *              dahash (containing indices into @da)
+ *              val  (searching for this number in @da)
+ *              &index (<return> index into da if found; -1 otherwise)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) Algo: hash @val into a key; hash the key to get the dna
+ *                in @dahash (that holds indices into @da); traverse
+ *                the dna of indices looking for @val in @da.
+ */
+l_int32
+l_dnaFindValByHash(L_DNA      *da,
+                   L_DNAHASH  *dahash,
+                   l_float64   val,
+                   l_int32    *pindex)
+{
+l_int32    i, nbuckets, nvals, indexval;
+l_float64  vali;
+l_uint64   key;
+L_DNA     *da1;
+
+    PROCNAME("l_dnaFindValByHash");
+
+    if (!pindex)
+        return ERROR_INT("&index not defined", procName, 1);
+    *pindex = -1;
+    if (!da)
+        return ERROR_INT("da not defined", procName, 1);
+    if (!dahash)
+        return ERROR_INT("dahash not defined", procName, 1);
+
+    nbuckets = l_dnaHashGetCount(dahash);
+    l_hashFloat64ToUint64(nbuckets, val, &key);
+    da1 = l_dnaHashGetDna(dahash, key, L_NOCOPY);
+    if (!da1) return 0;
+
+        /* Run through da1, looking for this @val */
+    nvals = l_dnaGetCount(da1);
+    for (i = 0; i < nvals; i++) {
+        l_dnaGetIValue(da1, i, &indexval);
+        l_dnaGetDValue(da, indexval, &vali);
+        if (val == vali) {
+            *pindex = indexval;
+            return 0;
+        }
+    }
+
+    return 0;
+}
+
+
 /*----------------------------------------------------------------------*
  *                         Other Dna functions                          *
  *----------------------------------------------------------------------*/
@@ -1646,7 +2221,7 @@ L_DNA     *da;
 /*!
  *  l_dnaJoin()
  *
- *      Input:  dad  (dest dma; add to this one)
+ *      Input:  dad  (dest dna; add to this one)
  *              das  (<optional> source dna; add from this one)
  *              istart  (starting index in das)
  *              iend  (ending index in das; use -1 to cat all)
@@ -1688,3 +2263,5 @@ l_float64  val;
 
     return 0;
 }
+
+
