@@ -99,10 +99,10 @@
 #define   TEMPLATE1       "morphtemplate1.txt"
 #define   TEMPLATE2       "morphtemplate2.txt"
 
-#define   BUFFER_SIZE     512
-
 #define   PROTOARGS   "(l_uint32 *, l_int32, l_int32, l_int32, l_uint32 *, l_int32);"
 
+static const l_int32  L_BUF_SIZE = 512;
+
 static char * makeBarrelshiftString(l_int32 delx, l_int32 dely);
 static SARRAY * sarrayMakeInnerLoopDWACode(SEL *sel, l_int32 index);
 static SARRAY * sarrayMakeWplsCode(SEL *sel);
@@ -284,7 +284,7 @@ char    *str_doc1, *str_doc2, *str_doc3, *str_doc4;
 char    *str_def1, *str_def2, *str_proc1, *str_proc2;
 char    *str_dwa1, *str_low_dt, *str_low_ds, *str_low_ts;
 char    *str_low_tsp1, *str_low_dtp1;
-char     bigbuf[BUFFER_SIZE];
+char     bigbuf[L_BUF_SIZE];
 l_int32  i, nsels, nbytes, actstart, end, newstart;
 size_t   size;
 SARRAY  *sa1, *sa2, *sa3;
@@ -306,7 +306,7 @@ SARRAY  *sa1, *sa2, *sa3;
         return ERROR_INT("filestr not made", procName, 1);
     if ((sa2 = sarrayCreateLinesFromString(filestr, 1)) == NULL)
         return ERROR_INT("sa2 not made", procName, 1);
-    FREE(filestr);
+    LEPT_FREE(filestr);
 /*    sarrayWriteStream(stderr, sa2); */
 
         /* Make strings containing function call names */
@@ -339,7 +339,7 @@ SARRAY  *sa1, *sa2, *sa3;
     str_proc2 = stringNew(bigbuf);
     sprintf(bigbuf,
             "    pixt2 = pixFMorphopGen_%d(NULL, pixt1, operation, selname);",
-	    fileindex);
+            fileindex);
     str_dwa1 = stringNew(bigbuf);
     sprintf(bigbuf,
       "            fmorphopgen_low_%d(datad, w, h, wpld, datat, wpls, index);",
@@ -389,12 +389,12 @@ SARRAY  *sa1, *sa2, *sa3;
     sprintf(bigbuf, "static char  SEL_NAMES[][80] = {");
     sarrayAddString(sa3, bigbuf, L_COPY);
     for (i = 0; i < nsels - 1; i++) {
-        sprintf(bigbuf,
-           "                             \"%s\",", sarrayGetString(sa1, i, 0));
+        sprintf(bigbuf, "                             \"%s\",",
+                sarrayGetString(sa1, i, L_NOCOPY));
         sarrayAddString(sa3, bigbuf, L_COPY);
     }
-    sprintf(bigbuf,
-        "                             \"%s\"};", sarrayGetString(sa1, i, 0));
+    sprintf(bigbuf, "                             \"%s\"};",
+            sarrayGetString(sa1, i, L_NOCOPY));
     sarrayAddString(sa3, bigbuf, L_COPY);
 
         /* Start pixMorphDwa_*() function description */
@@ -458,7 +458,7 @@ SARRAY  *sa1, *sa2, *sa3;
     sarrayDestroy(&sa1);
     sarrayDestroy(&sa2);
     sarrayDestroy(&sa3);
-    FREE(filestr);
+    LEPT_FREE(filestr);
     return 0;
 }
 
@@ -489,7 +489,7 @@ fmorphautogen2(SELA        *sela,
 {
 char    *filestr, *linestr, *fname;
 char    *str_doc1, *str_doc2, *str_doc3, *str_doc4, *str_def1;
-char     bigbuf[BUFFER_SIZE];
+char     bigbuf[L_BUF_SIZE];
 char     breakstring[] = "        break;";
 char     staticstring[] = "static void";
 l_int32  i, nsels, nbytes, actstart, end, newstart;
@@ -512,16 +512,16 @@ SEL     *sel;
         return ERROR_INT("filestr not made", procName, 1);
     if ((sa1 = sarrayCreateLinesFromString(filestr, 1)) == NULL)
         return ERROR_INT("sa1 not made", procName, 1);
-    FREE(filestr);
+    LEPT_FREE(filestr);
 
         /* Make the array of static function names */
     if ((sa2 = sarrayCreate(2 * nsels)) == NULL)
         return ERROR_INT("sa2 not made", procName, 1);
     for (i = 0; i < nsels; i++) {
         sprintf(bigbuf, "fdilate_%d_%d", fileindex, i);
-        sarrayAddString(sa2, bigbuf, 1);
+        sarrayAddString(sa2, bigbuf, L_COPY);
         sprintf(bigbuf, "ferode_%d_%d", fileindex, i);
-        sarrayAddString(sa2, bigbuf, 1);
+        sarrayAddString(sa2, bigbuf, L_COPY);
     }
 
         /* Make the static prototype strings */
@@ -530,7 +530,7 @@ SEL     *sel;
     for (i = 0; i < 2 * nsels; i++) {
         fname = sarrayGetString(sa2, i, 0);
         sprintf(bigbuf, "static void  %s%s", fname, PROTOARGS);
-        sarrayAddString(sa3, bigbuf, 1);
+        sarrayAddString(sa3, bigbuf, L_COPY);
     }
 
         /* Make strings containing function names */
@@ -605,7 +605,7 @@ SEL     *sel;
 
         /* Do all the static functions */
     for (i = 0; i < 2 * nsels; i++) {
-	    /* Generate the function header and add the common args */
+            /* Generate the function header and add the common args */
         sarrayAddString(sa4, staticstring, L_COPY);
         fname = sarrayGetString(sa2, i, L_NOCOPY);
         sprintf(bigbuf, "%s(l_uint32  *datad,", fname);
@@ -617,19 +617,19 @@ SEL     *sel;
             return ERROR_INT("sel not returned", procName, 1);
         if ((sa5 = sarrayMakeWplsCode(sel)) == NULL)
             return ERROR_INT("sa5 not made", procName, 1);
-        sarrayConcatenate(sa4, sa5);
+        sarrayJoin(sa4, sa5);
         sarrayDestroy(&sa5);
 
-	    /* Add the function loop code */
+            /* Add the function loop code */
         sarrayAppendRange(sa4, sa1, loopstart, loopend);
 
             /* Insert barrel-op code for *dptr */
         if ((sa6 = sarrayMakeInnerLoopDWACode(sel, i)) == NULL)
             return ERROR_INT("sa6 not made", procName, 1);
-        sarrayConcatenate(sa4, sa6);
+        sarrayJoin(sa4, sa6);
         sarrayDestroy(&sa6);
 
-	    /* Finish the function code */
+            /* Finish the function code */
         sarrayAppendRange(sa4, sa1, finalstart, finalend);
     }
 
@@ -646,7 +646,7 @@ SEL     *sel;
     sarrayDestroy(&sa2);
     sarrayDestroy(&sa3);
     sarrayDestroy(&sa4);
-    FREE(filestr);
+    LEPT_FREE(filestr);
 
     return 0;
 }
@@ -694,7 +694,7 @@ SARRAY  *sa;
     for (i = 0; i < ymax; i++) {
         if (vshift[i] == 0) {
             allvshifts = FALSE;
-	    break;
+            break;
         }
     }
 
@@ -704,34 +704,34 @@ SARRAY  *sa;
         /* Add declarations */
     if (allvshifts == TRUE) {   /* packs them as well as possible */
         if (ymax > 4)
-            sarrayAddString(sa, wpldecls[2], 1);
+            sarrayAddString(sa, wpldecls[2], L_COPY);
         if (ymax > 8)
-            sarrayAddString(sa, wpldecls[6], 1);
+            sarrayAddString(sa, wpldecls[6], L_COPY);
         if (ymax > 12)
-            sarrayAddString(sa, wpldecls[10], 1);
+            sarrayAddString(sa, wpldecls[10], L_COPY);
         if (ymax > 16)
-            sarrayAddString(sa, wpldecls[14], 1);
+            sarrayAddString(sa, wpldecls[14], L_COPY);
         if (ymax > 20)
-            sarrayAddString(sa, wpldecls[18], 1);
+            sarrayAddString(sa, wpldecls[18], L_COPY);
         if (ymax > 24)
-            sarrayAddString(sa, wpldecls[22], 1);
+            sarrayAddString(sa, wpldecls[22], L_COPY);
         if (ymax > 28)
-            sarrayAddString(sa, wpldecls[26], 1);
+            sarrayAddString(sa, wpldecls[26], L_COPY);
         if (ymax > 1)
-            sarrayAddString(sa, wpldecls[ymax - 2], 1);
+            sarrayAddString(sa, wpldecls[ymax - 2], L_COPY);
     } else {  /* puts them one/line */
         for (i = 2; i <= ymax; i++) {
             if (vshift[i])
-                sarrayAddString(sa, wplgendecls[i - 2], 1);
+                sarrayAddString(sa, wplgendecls[i - 2], L_COPY);
         }
     }
 
-    sarrayAddString(sa, emptystring, 1);
+    sarrayAddString(sa, emptystring, L_COPY);
 
         /* Add definitions */
     for (i = 2; i <= ymax; i++) {
         if (vshift[i])
-            sarrayAddString(sa, wpldefs[i - 2], 1);
+            sarrayAddString(sa, wpldefs[i - 2], L_COPY);
     }
 
     return sa;
@@ -748,7 +748,7 @@ sarrayMakeInnerLoopDWACode(SEL     *sel,
 char    *tstr, *string;
 char     logicalor[] = "|";
 char     logicaland[] = "&";
-char     bigbuf[BUFFER_SIZE];
+char     bigbuf[L_BUF_SIZE];
 l_int32  i, j, optype, count, nfound, delx, dely;
 SARRAY  *sa;
 
@@ -804,8 +804,8 @@ SARRAY  *sa;
                     sprintf(bigbuf, "                    %s %s", string, tstr);
                 else  /* nfound == count */
                     sprintf(bigbuf, "                    %s;", string);
-                sarrayAddString(sa, bigbuf, 1);
-                FREE(string);
+                sarrayAddString(sa, bigbuf, L_COPY);
+                LEPT_FREE(string);
             }
         }
     }
@@ -822,7 +822,7 @@ makeBarrelshiftString(l_int32  delx,    /* j - cx */
                       l_int32  dely)    /* i - cy */
 {
 l_int32  absx, absy;
-char     bigbuf[BUFFER_SIZE];
+char     bigbuf[L_BUF_SIZE];
 
     PROCNAME("makeBarrelshiftString");
 
