@@ -403,8 +403,8 @@ NUMAA    *naa_objs;  /* object mapping numbers to new values */
         }
         if (i == npages - 1)  /* last one */
             l_dnaAddNumber(da_outlocs, l_byteaGetSize(bad));
-        FREE(sizes);
-        FREE(locs);
+        LEPT_FREE(sizes);
+        LEPT_FREE(locs);
         l_dnaDestroy(&da_locs);
         numaDestroy(&na_objs);
         l_dnaDestroy(&da_sizes);
@@ -433,8 +433,8 @@ NUMAA    *naa_objs;  /* object mapping numbers to new values */
     numaaDestroy(&naa_objs);
     l_dnaDestroy(&da_outlocs);
     l_dnaaDestroy(&daa_locs);
-    FREE(str_pages);
-    FREE(str_trailer);
+    LEPT_FREE(str_pages);
+    LEPT_FREE(str_trailer);
     return 0;
 }
 
@@ -446,7 +446,7 @@ NUMAA    *naa_objs;  /* object mapping numbers to new values */
  *  l_generateCIDataForPdf()
  *
  *      Input:  fname
- *              pix <optional; can be null)
+ *              pix (<optional>; can be null)
  *              quality (for jpeg if transcoded; 75 is standard)
  *              &cid (<return> compressed data)
  *      Return: 0 if OK, 1 on error
@@ -489,14 +489,14 @@ PIX          *pixt;
     } else if (format == IFF_JP2) {
         cid = l_generateJp2kData(fname);
     } else if (format == IFF_PNG) {  /* use Jeff's special function for png */
-        cid = l_generateFlateDataPdf(fname);
+        cid = l_generateFlateDataPdf(fname, pix);
     } else {  /* any other format ... */
         if (!pix)
             pixt = pixRead(fname);
         else
             pixt = pixClone(pix);
         if (!pixt)
-            return ERROR_INT("fname not defined", procName, 1);
+            return ERROR_INT("pixt not made", procName, 1);
         selectDefaultPdfEncoding(pixt, &type);
         pixGenerateCIData(pixt, type, quality, 0, &cid);
         pixDestroy(&pixt);
@@ -514,15 +514,21 @@ PIX          *pixt;
  *  l_generateFlateDataPdf()
  *
  *      Input:  fname (preferably png)
+ *              pix (<optional>; can be null)
  *      Return: cid (containing png data), or null on error
  *
  *  Notes:
  *      (1) If you hand this a png file, you are going to get
  *          png predictors embedded in the flate data. So it has
  *          come to this. http://xkcd.com/1022/
+ *      (2) Exception: if the png is interlaced or if it is RGBA,
+ *          it will be transcoded.
+ *      (3) If transcoding is required, this will not have to read from
+ *          file if you also input a pix.
  */
 L_COMP_DATA *
-l_generateFlateDataPdf(const char  *fname)
+l_generateFlateDataPdf(const char  *fname,
+                       PIX         *pixs)
 {
 l_uint8      *pngcomp = NULL;  /* entire PNG compressed file */
 l_uint8      *datacomp = NULL;  /* gzipped raster data */
@@ -538,7 +544,7 @@ l_int32       xres, yres;
 size_t        nbytescomp = 0, nbytespng = 0;
 FILE         *fp;
 L_COMP_DATA  *cid;
-PIX          *pixs;
+PIX          *pix;
 PIXCMAP      *cmap = NULL;
 
     PROCNAME("l_generateFlateDataPdf");
@@ -547,15 +553,31 @@ PIXCMAP      *cmap = NULL;
         return (L_COMP_DATA *)ERROR_PTR("fname not defined", procName, NULL);
 
     findFileFormat(fname, &format);
-    if (format == IFF_PNG)
+    spp = 0;  /* init to spp != 4 if not png */
+    interlaced = 0;  /* initialize to no interlacing */
+    if (format == IFF_PNG) {
         isPngInterlaced(fname, &interlaced);
+        readHeaderPng(fname, NULL, NULL, NULL, &spp, NULL);
+    }
 
-        /* If either interlaced png or another format, transcode to flate */
-    if (interlaced || format != IFF_PNG) {
-        if ((pixs = pixRead(fname)) == NULL)
-            return (L_COMP_DATA *)ERROR_PTR("pixs not made", procName, NULL);
-        cid = pixGenerateFlateData(pixs, 0);
-        pixDestroy(&pixs);
+        /* PDF is capable of inlining some types of PNG files, but not all
+           of them. We need to transcode anything with interlacing or an
+           alpha channel.
+
+           Be careful with spp. Any PNG image file with an alpha
+           channel is converted on reading to RGBA (spp == 4). This
+           includes the (gray + alpha) format with spp == 2. You
+           will get different results if you look at spp via
+           readHeaderPng() versus pixGetSpp() */
+    if (format != IFF_PNG || interlaced || spp == 4 || spp == 2) {
+        if (!pixs)
+            pix = pixRead(fname);
+        else
+            pix = pixClone(pixs);
+        if (!pix)
+            return (L_COMP_DATA *)ERROR_PTR("pix not made", procName, NULL);
+        cid = pixGenerateFlateData(pix, 0);
+        pixDestroy(&pix);
         return cid;
     }
 
@@ -583,7 +605,7 @@ PIXCMAP      *cmap = NULL;
          * the png file, so after extraction we expect datacomp to
          * be nearly full (i.e., nbytescomp will be only slightly less
          * than nbytespng).  Also extract the colormap if present. */
-    if ((datacomp = (l_uint8 *)CALLOC(1, nbytespng)) == NULL)
+    if ((datacomp = (l_uint8 *)LEPT_CALLOC(1, nbytespng)) == NULL)
         return (L_COMP_DATA *)ERROR_PTR("unable to allocate memory",
                                         procName, NULL);
 
@@ -604,8 +626,8 @@ PIXCMAP      *cmap = NULL;
         n += pngcomp[i - 6] << 8;
         n += pngcomp[i - 5] << 0;
         if (i + n >= nbytespng) {
-            FREE(pngcomp);
-            FREE(datacomp);
+            LEPT_FREE(pngcomp);
+            LEPT_FREE(datacomp);
             pixcmapDestroy(&cmap);
             L_ERROR("invalid png: i = %d, n = %d, nbytes = %lu\n", procName,
                     i, n, (unsigned long)nbytespng);
@@ -622,8 +644,8 @@ PIXCMAP      *cmap = NULL;
         if (cmapflag && !cmap &&
             strncmp((const char *)(pngcomp + i - 4), "PLTE", 4) == 0) {
             if ((n / 3) > (1 << bps)) {
-                FREE(pngcomp);
-                FREE(datacomp);
+                LEPT_FREE(pngcomp);
+                LEPT_FREE(datacomp);
                 pixcmapDestroy(&cmap);
                 L_ERROR("invalid png: i = %d, n = %d, cmapsize = %d\n",
                         procName, i, n, (1 << bps));
@@ -637,10 +659,10 @@ PIXCMAP      *cmap = NULL;
         }
         i += n;  /* move to the end of the data chunk */
     }
-    FREE(pngcomp);
+    LEPT_FREE(pngcomp);
 
     if (nbytescomp == 0) {
-        FREE(datacomp);
+        LEPT_FREE(datacomp);
         pixcmapDestroy(&cmap);
         return (L_COMP_DATA *)ERROR_PTR("invalid PNG file", procName, NULL);
     }
@@ -651,12 +673,12 @@ PIXCMAP      *cmap = NULL;
         pixcmapSerializeToMemory(cmap, 3, &ncolors, &cmapdata);
         pixcmapDestroy(&cmap);
         if (!cmapdata) {
-            FREE(datacomp);
+            LEPT_FREE(datacomp);
             return (L_COMP_DATA *)ERROR_PTR("cmapdata not made",
                                             procName, NULL);
         }
         cmapdatahex = pixcmapConvertToHex(cmapdata, ncolors);
-        FREE(cmapdata);
+        LEPT_FREE(cmapdata);
     }
 
         /* Note that this is the only situation where the predictor
@@ -666,7 +688,7 @@ PIXCMAP      *cmap = NULL;
          * ignored by the pdf interpreter, which just needs to know that
          * the first byte on each compressed scanline is some predictor
          * whose type can be inferred from the byte itself.  */
-    cid = (L_COMP_DATA *)CALLOC(1, sizeof(L_COMP_DATA));
+    cid = (L_COMP_DATA *)LEPT_CALLOC(1, sizeof(L_COMP_DATA));
     cid->datacomp = datacomp;
     cid->type = L_FLATE_ENCODE;
     cid->cmapdatahex = cmapdatahex;
@@ -729,14 +751,14 @@ L_COMP_DATA  *cid;
         /* Optionally, encode the compressed data */
     if (ascii85flag == 1) {
         data85 = encodeAscii85(datacomp, nbytescomp, &nbytes85);
-        FREE(datacomp);
+        LEPT_FREE(datacomp);
         if (!data85)
             return (L_COMP_DATA *)ERROR_PTR("data85 not made", procName, NULL);
         else
             data85[nbytes85 - 1] = '\0';  /* remove the newline */
     }
 
-    cid = (L_COMP_DATA *)CALLOC(1, sizeof(L_COMP_DATA));
+    cid = (L_COMP_DATA *)LEPT_CALLOC(1, sizeof(L_COMP_DATA));
     if (!cid)
         return (L_COMP_DATA *)ERROR_PTR("cid not made", procName, NULL);
     if (ascii85flag == 0) {
@@ -777,7 +799,7 @@ L_COMP_DATA  *cid;
     if (!fname)
         return (L_COMP_DATA *)ERROR_PTR("fname not defined", procName, NULL);
 
-    if ((cid = (L_COMP_DATA *)CALLOC(1, sizeof(L_COMP_DATA))) == NULL)
+    if ((cid = (L_COMP_DATA *)LEPT_CALLOC(1, sizeof(L_COMP_DATA))) == NULL)
         return (L_COMP_DATA *)ERROR_PTR("cid not made", procName, NULL);
 
         /* The returned jp2k data in memory is the entire jp2k file */
@@ -1007,6 +1029,13 @@ PIX          *pixs;
  *      Input:  pixs
  *              ascii85flag (0 for gzipped; 1 for ascii85-encoded gzipped)
  *      Return: cid (flate compressed image data), or null on error
+ *
+ *      Notes:
+ *          (1) This should not be called with an RGBA pix (spp == 4); it
+ *              will ignore the alpha channel.  Likewise, if called with a
+ *              colormapped pix, the alpha component in the colormap will
+ *              be ignored (as it is for all leptonica operations
+ *              on colormapped pix).
  */
 static L_COMP_DATA *
 pixGenerateFlateData(PIX     *pixs,
@@ -1020,7 +1049,7 @@ char         *cmapdata85 = NULL;  /* ascii85 encoded uncompressed colormap */
 char         *cmapdatahex = NULL;  /* hex ascii uncompressed colormap */
 l_int32       ncolors;  /* in colormap; not used if cmapdata85 is null */
 l_int32       bps;  /* bits/sample: usually 8 */
-l_int32       spp;  /* samples/pixel: 1-grayscale/cmap); 3-rgb; 4-rgba */
+l_int32       spp;  /* samples/pixel: 1-grayscale/cmap); 3-rgb */
 l_int32       w, h, d, cmapflag;
 l_int32       ncmapbytes85 = 0;
 l_int32       nbytes85 = 0;
@@ -1049,7 +1078,7 @@ PIXCMAP      *cmap;
     } else {
         pixt = pixClone(pixs);
     }
-    spp = (d == 32) ? 3 : 1;
+    spp = (d == 32) ? 3 : 1;  /* ignores alpha */
     bps = (d == 32) ? 8 : d;
 
         /* Extract and encode the colormap data as both ascii85 and hexascii  */
@@ -1062,7 +1091,7 @@ PIXCMAP      *cmap;
 
         cmapdata85 = encodeAscii85(cmapdata, 3 * ncolors, &ncmapbytes85);
         cmapdatahex = pixcmapConvertToHex(cmapdata, ncolors);
-        FREE(cmapdata);
+        LEPT_FREE(cmapdata);
     }
 
         /* Extract and compress the raster data */
@@ -1070,25 +1099,25 @@ PIXCMAP      *cmap;
     pixDestroy(&pixt);
     datacomp = zlibCompress(data, nbytes, &nbytescomp);
     if (!datacomp) {
-        if (cmapdata85) FREE(cmapdata85);
-        if (cmapdatahex) FREE(cmapdatahex);
+        if (cmapdata85) LEPT_FREE(cmapdata85);
+        if (cmapdatahex) LEPT_FREE(cmapdatahex);
         return (L_COMP_DATA *)ERROR_PTR("datacomp not made", procName, NULL);
     }
-    FREE(data);
+    LEPT_FREE(data);
 
         /* Optionally, encode the compressed data */
     if (ascii85flag == 1) {
         data85 = encodeAscii85(datacomp, nbytescomp, &nbytes85);
-        FREE(datacomp);
+        LEPT_FREE(datacomp);
         if (!data85) {
-            FREE(cmapdata85);
+            LEPT_FREE(cmapdata85);
             return (L_COMP_DATA *)ERROR_PTR("data85 not made", procName, NULL);
         } else {
             data85[nbytes85 - 1] = '\0';  /* remove the newline */
         }
     }
 
-    cid = (L_COMP_DATA *)CALLOC(1, sizeof(L_COMP_DATA));
+    cid = (L_COMP_DATA *)LEPT_CALLOC(1, sizeof(L_COMP_DATA));
     if (!cid)
         return (L_COMP_DATA *)ERROR_PTR("cid not made", procName, NULL);
     if (ascii85flag == 0) {
@@ -1243,14 +1272,14 @@ FILE         *fp;
         /* Optionally, encode the compressed data */
     if (ascii85flag == 1) {
         data85 = encodeAscii85(datacomp, nbytescomp, &nbytes85);
-        FREE(datacomp);
+        LEPT_FREE(datacomp);
         if (!data85)
             return (L_COMP_DATA *)ERROR_PTR("data85 not made", procName, NULL);
         else
             data85[nbytes85 - 1] = '\0';  /* remove the newline */
     }
 
-    cid = (L_COMP_DATA *)CALLOC(1, sizeof(L_COMP_DATA));
+    cid = (L_COMP_DATA *)LEPT_CALLOC(1, sizeof(L_COMP_DATA));
     if (!cid)
         return (L_COMP_DATA *)ERROR_PTR("cid not made", procName, NULL);
     if (ascii85flag == 0) {
@@ -1347,11 +1376,11 @@ L_COMP_DATA  *cid;
     if ((cid = *pcid) == NULL)
         return;
 
-    if (cid->datacomp) FREE(cid->datacomp);
-    if (cid->data85) FREE(cid->data85);
-    if (cid->cmapdata85) FREE(cid->cmapdata85);
-    if (cid->cmapdatahex) FREE(cid->cmapdatahex);
-    FREE(cid);
+    if (cid->datacomp) LEPT_FREE(cid->datacomp);
+    if (cid->data85) LEPT_FREE(cid->data85);
+    if (cid->cmapdata85) LEPT_FREE(cid->cmapdata85);
+    if (cid->cmapdatahex) LEPT_FREE(cid->cmapdatahex);
+    LEPT_FREE(cid);
     *pcid = NULL;
     return;
 }
@@ -1432,11 +1461,11 @@ SARRAY  *sa;
         datestr = l_getFormattedDate();
         snprintf(buf, sizeof(buf), "/CreationDate (D:%s)\n", datestr);
         sarrayAddString(sa, (char *)buf, L_COPY);
-        FREE(datestr);
+        LEPT_FREE(datestr);
         version = getLeptonicaVersion();
         snprintf(buf, sizeof(buf),
                  "/Producer (leptonica: %s)\n", version);
-        FREE(version);
+        LEPT_FREE(version);
     } else {
         snprintf(buf, sizeof(buf), "/Producer (leptonica)\n");
     }
@@ -1514,7 +1543,7 @@ SARRAY  *sa;
          * 50 bytes for each reference to an image in the
          * ProcSet array.  */
     bufsize = 1000 + 50 * lpd->n;
-    if ((buf = (char *)CALLOC(bufsize, sizeof(char))) == NULL)
+    if ((buf = (char *)LEPT_CALLOC(bufsize, sizeof(char))) == NULL)
         return ERROR_INT("calloc fail for buf", procName, 1);
 
     boxGetGeometry(lpd->mediabox, NULL, NULL, &wpt, &hpt);
@@ -1545,8 +1574,8 @@ SARRAY  *sa;
     lpd->obj4 = stringNew(buf);
     l_dnaAddNumber(lpd->objsize, strlen(lpd->obj4));
     sarrayDestroy(&sa);
-    FREE(buf);
-    FREE(xstr);
+    LEPT_FREE(buf);
+    LEPT_FREE(xstr);
     return 0;
 }
 
@@ -1563,7 +1592,7 @@ SARRAY    *sa;
     PROCNAME("generateContentStringPdf");
 
     bufsize = 1000 + 200 * lpd->n;
-    if ((buf = (char *)CALLOC(bufsize, sizeof(char))) == NULL)
+    if ((buf = (char *)LEPT_CALLOC(bufsize, sizeof(char))) == NULL)
         return ERROR_INT("calloc fail for buf", procName, 1);
 
     sa = sarrayCreate(lpd->n);
@@ -1590,8 +1619,8 @@ SARRAY    *sa;
     lpd->obj5 = stringNew(buf);
     l_dnaAddNumber(lpd->objsize, strlen(lpd->obj5));
     sarrayDestroy(&sa);
-    FREE(buf);
-    FREE(cstr);
+    LEPT_FREE(buf);
+    LEPT_FREE(cstr);
     return 0;
 }
 
@@ -1611,7 +1640,7 @@ SARRAY       *sa;
     sa = lpd->saprex;
     cmindex = 6 + lpd->n;  /* starting value */
     for (i = 0; i < lpd->n; i++) {
-        pstr = NULL;
+        pstr = cstr = NULL;
         if ((cid = pdfdataGetCid(lpd, i)) == NULL)
             return ERROR_INT("cid not found", procName, 1);
 
@@ -1638,7 +1667,7 @@ SARRAY       *sa;
             else if (cid->spp == 3)
                 cstr = stringNew("/ColorSpace /DeviceRGB");
             else
-                L_ERROR("spp!= 1 && spp != 3\n", procName);
+                L_ERROR("in jpeg: spp != 1 && spp != 3\n", procName);
             bstr = stringNew("/BitsPerComponent 8");
             fstr = stringNew("/Filter /DCTDecode");
         } else if (cid->type == L_JP2K_ENCODE) {
@@ -1647,7 +1676,7 @@ SARRAY       *sa;
             else if (cid->spp == 3)
                 cstr = stringNew("/ColorSpace /DeviceRGB");
             else
-                L_ERROR("spp!= 1 && spp != 3\n", procName);
+                L_ERROR("in jp2k: spp != 1 && spp != 3\n", procName);
             bstr = stringNew("/BitsPerComponent 8");
             fstr = stringNew("/Filter /JPXDecode");
         } else {  /* type == L_FLATE_ENCODE */
@@ -1663,7 +1692,8 @@ SARRAY       *sa;
                 else if (cid->spp == 3)
                     cstr = stringNew("/ColorSpace /DeviceRGB");
                 else
-                    L_ERROR("unknown colorspace\n", procName);
+                    L_ERROR("unknown colorspace: spp = %d\n",
+                            procName, cid->spp);
             }
             snprintf(buff, sizeof(buff), "/BitsPerComponent %d", cid->bps);
             bstr = stringNew(buff);
@@ -1674,8 +1704,9 @@ SARRAY       *sa;
                          "<<\n"
                          "  /Columns %d\n"
                          "  /Predictor 14\n"
+                         "  /Colors %d\n"
                          "  /BitsPerComponent %d\n"
-                         ">>\n", cid->w, cid->bps);
+                         ">>\n", cid->w, cid->spp, cid->bps);
                 pstr = stringNew(buff);
             }
         }
@@ -1701,10 +1732,10 @@ SARRAY       *sa;
         sarrayAddString(sa, xstr, L_INSERT);
         l_dnaAddNumber(lpd->objsize,
                       strlen(xstr) + cid->nbytescomp + strlen(lpd->poststream));
-        FREE(cstr);
-        FREE(bstr);
-        FREE(fstr);
-        FREE(pstr);
+        LEPT_FREE(cstr);
+        LEPT_FREE(bstr);
+        LEPT_FREE(fstr);
+        LEPT_FREE(pstr);
     }
 
     return 0;
@@ -1867,7 +1898,7 @@ L_COMP_DATA  *cid;
         return ERROR_INT("&nbytes not defined", procName, 1);
     nbytes = lpd->xrefloc + strlen(lpd->trailer);
     *pnbytes = nbytes;
-    if ((data = (l_uint8 *)CALLOC(nbytes, sizeof(l_uint8))) == NULL)
+    if ((data = (l_uint8 *)LEPT_CALLOC(nbytes, sizeof(l_uint8))) == NULL)
         return ERROR_INT("calloc fail for data", procName, 1);
     *pdata = data;
 
@@ -1903,8 +1934,8 @@ L_COMP_DATA  *cid;
 
         /* And finally the trailer */
     memcpy((char *)(data + lpd->xrefloc), lpd->trailer, strlen(lpd->trailer));
-    FREE(sizes);
-    FREE(locs);
+    LEPT_FREE(sizes);
+    LEPT_FREE(locs);
     return 0;
 }
 
@@ -2032,7 +2063,7 @@ SARRAY  *sa;
 
     n = numaGetCount(napage);
     bufsize = 100 + 16 * n;  /* large enough to hold the output string */
-    buf = (char *)CALLOC(bufsize, sizeof(char));
+    buf = (char *)LEPT_CALLOC(bufsize, sizeof(char));
     sa = sarrayCreate(n);
     for (i = 0; i < n; i++) {
         numaGetIValue(napage, i, &index);
@@ -2048,7 +2079,7 @@ SARRAY  *sa;
                                "/Count %d\n"
                                ">>\n", str, n);
     sarrayDestroy(&sa);
-    FREE(str);
+    LEPT_FREE(str);
     return buf;
 }
 
@@ -2064,7 +2095,7 @@ SARRAY  *sa;
  *      (1) Interpret the first set of bytes as the object number,
  *          map to the new number, and write it out.
  *      (2) Find all occurrences of this 4-byte sequence: " 0 R"
- *      (3) Find the location and value of the integer preceeding this,
+ *      (3) Find the location and value of the integer preceding this,
  *          and map it to the new value.
  *      (4) Rewrite the object with new object numbers.
  */
@@ -2096,7 +2127,7 @@ L_DNA    *da_match;
     da_match = arrayFindEachSequence(datas, size, (l_uint8 *)" 0 R", 4);
     if (!da_match) {
         l_byteaAppendData(bad, datas + start, size - start);
-        FREE(objs);
+        LEPT_FREE(objs);
         return bad;
     }
 
@@ -2119,8 +2150,8 @@ L_DNA    *da_match;
     }
     l_byteaAppendData(bad, datas + start, size - start);
 
-    FREE(objs);
-    FREE(matches);
+    LEPT_FREE(objs);
+    LEPT_FREE(matches);
     l_dnaDestroy(&da_match);
     return bad;
 }
@@ -2134,7 +2165,7 @@ pdfdataCreate(const char  *title)
 {
 L_PDF_DATA *lpd;
 
-    lpd = (L_PDF_DATA *)CALLOC(1, sizeof(L_PDF_DATA));
+    lpd = (L_PDF_DATA *)LEPT_CALLOC(1, sizeof(L_PDF_DATA));
     if (title) lpd->title = stringNew(title);
     lpd->cida = ptraCreate(10);
     lpd->xy = ptaCreate(10);
@@ -2162,21 +2193,21 @@ L_PDF_DATA   *lpd;
     if ((lpd = *plpd) == NULL)
         return;
 
-    if (lpd->title) FREE(lpd->title);
+    if (lpd->title) LEPT_FREE(lpd->title);
     for (i = 0; i < lpd->n; i++) {
         cid = (L_COMP_DATA *)ptraRemove(lpd->cida, i, L_NO_COMPACTION);
         l_CIDataDestroy(&cid);
     }
 
     ptraDestroy(&lpd->cida, 0, 0);
-    if (lpd->id) FREE(lpd->id);
-    if (lpd->obj1) FREE(lpd->obj1);
-    if (lpd->obj2) FREE(lpd->obj2);
-    if (lpd->obj3) FREE(lpd->obj3);
-    if (lpd->obj4) FREE(lpd->obj4);
-    if (lpd->obj5) FREE(lpd->obj5);
-    if (lpd->poststream) FREE(lpd->poststream);
-    if (lpd->trailer) FREE(lpd->trailer);
+    if (lpd->id) LEPT_FREE(lpd->id);
+    if (lpd->obj1) LEPT_FREE(lpd->obj1);
+    if (lpd->obj2) LEPT_FREE(lpd->obj2);
+    if (lpd->obj3) LEPT_FREE(lpd->obj3);
+    if (lpd->obj4) LEPT_FREE(lpd->obj4);
+    if (lpd->obj5) LEPT_FREE(lpd->obj5);
+    if (lpd->poststream) LEPT_FREE(lpd->poststream);
+    if (lpd->trailer) LEPT_FREE(lpd->trailer);
     if (lpd->xy) ptaDestroy(&lpd->xy);
     if (lpd->wh) ptaDestroy(&lpd->wh);
     if (lpd->mediabox) boxDestroy(&lpd->mediabox);
@@ -2184,7 +2215,7 @@ L_PDF_DATA   *lpd;
     if (lpd->sacmap) sarrayDestroy(&lpd->sacmap);
     if (lpd->objsize) l_dnaDestroy(&lpd->objsize);
     if (lpd->objloc) l_dnaDestroy(&lpd->objloc);
-    FREE(lpd);
+    LEPT_FREE(lpd);
     *plpd = NULL;
     return;
 }
