@@ -184,7 +184,7 @@ l_uint32  *line, *data;
     PROCNAME("pixGetPixel");
 
     if (!pval)
-        return ERROR_INT("pval not defined", procName, 1);
+        return ERROR_INT("&val not defined", procName, 1);
     *pval = 0;
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
@@ -256,7 +256,6 @@ l_uint32  *line, *data;
 
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
-
     pixGetDimensions(pix, &w, &h, &d);
     if (x < 0 || x >= w)
         return ERROR_INT("x out of bounds", procName, 1);
@@ -320,6 +319,11 @@ l_uint32  *data, *ppixel;
 
     PROCNAME("pixGetRGBPixel");
 
+    if (prval) *prval = 0;
+    if (pgval) *pgval = 0;
+    if (pbval) *pbval = 0;
+    if (!prval && !pgval && !pbval)
+        return ERROR_INT("no output requested", procName, 1);
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
     pixGetDimensions(pix, &w, &h, &d);
@@ -387,7 +391,7 @@ l_uint32  *data, *line;
  *  pixGetRandomPixel()
  *
  *      Input:  pix (any depth; can be colormapped)
- *              &val (<return> pixel value)
+ *              &val (<optional return> pixel value)
  *              &x (<optional return> x coordinate chosen; can be null)
  *              &y (<optional return> y coordinate chosen; can be null)
  *      Return: 0 if OK; 1 on error
@@ -407,9 +411,11 @@ PIXCMAP  *cmap;
 
     PROCNAME("pixGetRandomPixel");
 
-    if (!pval)
-        return ERROR_INT("pval not defined", procName, 1);
-    *pval = 0;
+    if (pval) *pval = 0;
+    if (px) *px = 0;
+    if (py) *py = 0;
+    if (!pval && !px && !py)
+        return ERROR_INT("no output requested", procName, 1);
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
 
@@ -418,12 +424,14 @@ PIXCMAP  *cmap;
     y = rand() % h;
     if (px) *px = x;
     if (py) *py = y;
-    pixGetPixel(pix, x, y, &val);
-    if ((cmap = pixGetColormap(pix)) != NULL) {
-        pixcmapGetColor(cmap, val, &rval, &gval, &bval);
-        composeRGBPixel(rval, gval, bval, pval);
-    } else {
-        *pval = val;
+    if (pval) {
+        pixGetPixel(pix, x, y, &val);
+        if ((cmap = pixGetColormap(pix)) != NULL) {
+            pixcmapGetColor(cmap, val, &rval, &gval, &bval);
+            composeRGBPixel(rval, gval, bval, pval);
+        } else {
+            *pval = val;
+        }
     }
 
     return 0;
@@ -449,7 +457,6 @@ l_uint32  *line, *data;
 
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
-
     pixGetDimensions(pix, &w, &h, &d);
     if (x < 0 || x >= w)
         return ERROR_INT("x out of bounds", procName, 1);
@@ -507,7 +514,6 @@ l_uint32  *line, *data;
 
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
-
     pixGetDimensions(pix, &w, &h, &d);
     if (x < 0 || x >= w)
         return ERROR_INT("x out of bounds", procName, 1);
@@ -1245,11 +1251,10 @@ l_uint32  *data, *pword;
 
     data = pixGetData(pix);
     wpl = pixGetWpl(pix);
-    endbits = 32 - ((w * d) % 32);
+    endbits = 32 - (((l_int64)w * d) % 32);
     if (endbits == 32)  /* no partial word */
         return 0;
-    fullwords = w * d / 32;
-
+    fullwords = (1LL * w * d) / 32;
     mask = rmask32[endbits];
     if (val == 0)
         mask = ~mask;
@@ -1313,10 +1318,10 @@ l_uint32  *data, *pword;
 
     data = pixGetData(pix);
     wpl = pixGetWpl(pix);
-    endbits = 32 - ((w * d) % 32);
+    endbits = 32 - (((l_int64)w * d) % 32);
     if (endbits == 32)  /* no partial word */
         return 0;
-    fullwords = w * d / 32;
+    fullwords = (l_int64)w * d / 32;
 
     mask = rmask32[endbits];
     if (val == 0)
@@ -3029,10 +3034,10 @@ l_uint32  *rline, *rdata;  /* data in pix raster */
 
     PROCNAME("pixGetRasterData");
 
+    if (pdata) *pdata = NULL;
+    if (pnbytes) *pnbytes = 0;
     if (!pdata || !pnbytes)
         return ERROR_INT("&data and &nbytes not both defined", procName, 1);
-    *pdata = NULL;
-    *pnbytes = 0;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     pixGetDimensions(pixs, &w, &h, &d);
@@ -3050,7 +3055,7 @@ l_uint32  *rline, *rdata;  /* data in pix raster */
         databpl = w * (d / 8);
     else  /* d == 32 bpp rgb */
         databpl = 3 * w;
-    if ((data = (l_uint8 *)CALLOC(databpl * h, sizeof(l_uint8))) == NULL)
+    if ((data = (l_uint8 *)LEPT_CALLOC(databpl * h, sizeof(l_uint8))) == NULL)
         return ERROR_INT("data not allocated", procName, 1);
     *pdata = data;
     *pnbytes = databpl * h;
@@ -3168,12 +3173,12 @@ l_int32  w, h;
     if (!pix || pixGetDepth(pix) != 8)
         return (l_uint8 **)ERROR_PTR("pix not defined or not 8 bpp",
                                      procName, NULL);
-    if (pixGetColormap(pix))
-        return (l_uint8 **)ERROR_PTR("pix has colormap", procName, NULL);
-
     pixGetDimensions(pix, &w, &h, NULL);
     if (pw) *pw = w;
     if (ph) *ph = h;
+    if (pixGetColormap(pix))
+        return (l_uint8 **)ERROR_PTR("pix has colormap", procName, NULL);
+
     pixEndianByteSwap(pix);
     return (l_uint8 **)pixGetLinePtrs(pix, NULL);
 }
@@ -3202,7 +3207,7 @@ pixCleanupByteProcessing(PIX      *pix,
         return ERROR_INT("lineptrs not defined", procName, 1);
 
     pixEndianByteSwap(pix);
-    FREE(lineptrs);
+    LEPT_FREE(lineptrs);
     return 0;
 }
 
