@@ -42,9 +42,8 @@
  *           PIX      *pixProjectivePtaGray()
  *           PIX      *pixProjectiveGray()
  *
- *      Projective transform including alpha (blend) component and gamma xform
+ *      Projective transform including alpha (blend) component
  *           PIX      *pixProjectivePtaWithAlpha()
- *           PIX      *pixProjectivePtaGammaXform()
  *
  *      Projective coordinate transformation
  *           l_int32   getProjectiveXformCoeffs()
@@ -82,12 +81,13 @@
  *
  *      where the eight coefficients have been computed from four
  *      sets of these equations, each for two corresponding data pts.
- *      In practice, for each point (x,y) in the dest image, this
- *      equation is used to compute the corresponding point (x',y')
- *      in the src.  That computed point in the src is then used
- *      to determine the dest value in one of two ways:
+ *      In practice, once the coefficients are known, we use the
+ *      equations "backwards": for each point (x,y) in the dest image,
+ *      these two equations are used to compute the corresponding point
+ *      (x',y') in the src.  That computed point in the src is then used
+ *      to determine the corresponding dest pixel value in one of two ways:
  *
- *       - sampling: take the value of the src pixel in which this
+ *       - sampling: simply take the value of the src pixel in which this
  *                   point falls
  *       - interpolation: take appropriate linear combinations of the
  *                        four src pixels that this dest pixel would
@@ -160,7 +160,7 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getProjectiveXformCoeffs(ptad, ptas, &vc);
     pixd = pixProjectiveSampled(pixs, vc, incolor);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
@@ -213,13 +213,13 @@ PIXCMAP    *cmap;
             color = 0;
         pixcmapAddBlackOrWhite(cmap, color, &cmapindex);
         pixSetAllArbitrary(pixd, cmapindex);
-    }
-    else {
+    } else {
         if ((d == 1 && incolor == L_BRING_IN_WHITE) ||
-            (d > 1 && incolor == L_BRING_IN_BLACK))
+            (d > 1 && incolor == L_BRING_IN_BLACK)) {
             pixClearAll(pixd);
-        else
+        } else {
             pixSetAll(pixd);
+        }
     }
 
         /* Scan over the dest pixels */
@@ -237,19 +237,15 @@ PIXCMAP    *cmap;
             if (d == 1) {
                 val = GET_DATA_BIT(lines, x);
                 SET_DATA_BIT_VAL(lined, j, val);
-            }
-            else if (d == 8) {
+            } else if (d == 8) {
                 val = GET_DATA_BYTE(lines, x);
                 SET_DATA_BYTE(lined, j, val);
-            }
-            else if (d == 32) {
+            } else if (d == 32) {
                 lined[j] = lines[x];
-            }
-            else if (d == 2) {
+            } else if (d == 2) {
                 val = GET_DATA_DIBIT(lines, x);
                 SET_DATA_DIBIT(lined, j, val);
-            }
-            else if (d == 4) {
+            } else if (d == 4) {
                 val = GET_DATA_QBIT(lines, x);
                 SET_DATA_QBIT(lined, j, val);
             }
@@ -427,7 +423,7 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getProjectiveXformCoeffs(ptad, ptas, &vc);
     pixd = pixProjectiveColor(pixs, vc, colorval);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
@@ -450,7 +446,7 @@ l_int32    i, j, w, h, d, wpls, wpld;
 l_uint32   val;
 l_uint32  *datas, *datad, *lined;
 l_float32  x, y;
-PIX       *pixd;
+PIX       *pix1, *pix2, *pixd;
 
     PROCNAME("pixProjectiveColor");
 
@@ -481,6 +477,15 @@ PIX       *pixd;
         }
     }
 
+        /* If rgba, transform the pixs alpha channel and insert in pixd */
+    if (pixGetSpp(pixs) == 4) {
+        pix1 = pixGetRGBComponent(pixs, L_ALPHA_CHANNEL);
+        pix2 = pixProjectiveGray(pix1, vc, 255);  /* bring in opaque */
+        pixSetRGBComponent(pixd, pix2, L_ALPHA_CHANNEL);
+        pixDestroy(&pix1);
+        pixDestroy(&pix2);
+    }
+
     return pixd;
 }
 
@@ -521,7 +526,7 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getProjectiveXformCoeffs(ptad, ptas, &vc);
     pixd = pixProjectiveGray(pixs, vc, grayval);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
@@ -579,7 +584,7 @@ PIX       *pixd;
 
 
 /*---------------------------------------------------------------------------*
- *   Projective transform including alpha (blend) component and gamma xform  *
+ *            Projective transform including alpha (blend) component         *
  *---------------------------------------------------------------------------*/
 /*!
  *  pixProjectivePtaWithAlpha()
@@ -643,15 +648,15 @@ PTA     *ptad2, *ptas2;
     if (d != 32 && pixGetColormap(pixs) == NULL)
         return (PIX *)ERROR_PTR("pixs not cmapped or 32 bpp", procName, NULL);
     if (pixg && pixGetDepth(pixg) != 8) {
-        L_WARNING("pixg not 8 bpp; using @fract transparent alpha", procName);
+        L_WARNING("pixg not 8 bpp; using @fract transparent alpha\n", procName);
         pixg = NULL;
     }
     if (!pixg && (fract < 0.0 || fract > 1.0)) {
-        L_WARNING("invalid fract; using 1.0 (fully transparent)", procName);
+        L_WARNING("invalid fract; using 1.0 (fully transparent)\n", procName);
         fract = 1.0;
     }
     if (!pixg && fract == 0.0)
-        L_WARNING("fully opaque alpha; image will not be blended", procName);
+        L_WARNING("fully opaque alpha; image will not be blended\n", procName);
     if (!ptad)
         return (PIX *)ERROR_PTR("ptad not defined", procName, NULL);
     if (!ptas)
@@ -673,9 +678,9 @@ PTA     *ptad2, *ptas2;
             pixSetAll(pixg2);
         else
             pixSetAllArbitrary(pixg2, (l_int32)(255.0 * fract));
-    }
-    else
+    } else {
         pixg2 = pixResizeToMatch(pixg, NULL, ws, hs);
+    }
     if (ws > 10 && hs > 10) {  /* see note 7 */
         pixSetBorderRingVal(pixg2, 1,
                             (l_int32)(255.0 * fract * AlphaMaskBorderVals[0]));
@@ -686,6 +691,7 @@ PTA     *ptad2, *ptas2;
     pixb2 = pixAddBorder(pixg2, border, 0);  /* must be black border */
     pixga = pixProjectivePtaGray(pixb2, ptad2, ptas2, 0);
     pixSetRGBComponent(pixd, pixga, L_ALPHA_CHANNEL);
+    pixSetSpp(pixd, 4);
 
     pixDestroy(&pixg2);
     pixDestroy(&pixb1);
@@ -697,59 +703,6 @@ PTA     *ptad2, *ptas2;
 }
 
 
-/*!
- *  pixProjectivePtaGammaXform()
- *
- *      Input:  pixs (32 bpp rgb)
- *              gamma (gamma correction; must be > 0.0)
- *              ptad  (3 pts of final coordinate space)
- *              ptas  (3 pts of initial coordinate space)
- *              fract (between 0.0 and 1.0, with 1.0 fully transparent)
- *              border (of pixels to capture transformed source pixels)
- *      Return: pixd, or null on error
- *
- *  Notes:
- *      (1) This wraps a gamma/inverse-gamma photometric transform around
- *          pixProjectivePtaWithAlpha().
- *      (2) For usage, see notes in pixProjectivePtaWithAlpha() and
- *          pixGammaTRCWithAlpha().
- *      (3) The basic idea of a gamma/inverse-gamma transform is to remove
- *          any gamma correction before the projective transform, and restore
- *          it afterward.  The effects can be subtle, but important for
- *          some applications.  For example, using gamma > 1.0 will
- *          cause the dark areas to become somewhat lighter and slightly
- *          reduce aliasing effects when blending using the alpha channel.
- */
-PIX *
-pixProjectivePtaGammaXform(PIX       *pixs,
-                           l_float32  gamma,
-                           PTA       *ptad,
-                           PTA       *ptas,
-                           l_float32  fract,
-                           l_int32    border)
-{
-PIX  *pixg, *pixd;
-
-    PROCNAME("pixProjectivePtaGammaXform");
-
-    if (!pixs || (pixGetDepth(pixs) != 32))
-        return (PIX *)ERROR_PTR("pixs undefined or not 32 bpp", procName, NULL);
-    if (fract == 0.0)
-        L_WARNING("fully opaque alpha; image cannot be blended", procName);
-    if (gamma <= 0.0)  {
-        L_WARNING("gamma must be > 0.0; setting to 1.0", procName);
-        gamma = 1.0;
-    }
-
-    pixg = pixGammaTRCWithAlpha(NULL, pixs, 1.0 / gamma, 0, 255);
-    pixd = pixProjectivePtaWithAlpha(pixg, ptad, ptas, NULL, fract, border);
-    pixGammaTRCWithAlpha(pixd, pixd, gamma, 0, 255);
-    pixDestroy(&pixg);
-    return pixd;
-}
-
-
-
 /*-------------------------------------------------------------*
  *                Projective coordinate transformation         *
  *-------------------------------------------------------------*/
@@ -824,7 +777,7 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     if (!pvc)
         return ERROR_INT("&vc not defined", procName, 1);
 
-    if ((b = (l_float32 *)CALLOC(8, sizeof(l_float32))) == NULL)
+    if ((b = (l_float32 *)LEPT_CALLOC(8, sizeof(l_float32))) == NULL)
         return ERROR_INT("b not made", procName, 1);
     *pvc = b;
 
@@ -838,7 +791,7 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     ptaGetPt(ptad, 3, &b[6], &b[7]);
 
     for (i = 0; i < 8; i++) {
-        if ((a[i] = (l_float32 *)CALLOC(8, sizeof(l_float32))) == NULL)
+        if ((a[i] = (l_float32 *)LEPT_CALLOC(8, sizeof(l_float32))) == NULL)
             return ERROR_INT("a[i] not made", procName, 1);
     }
 
@@ -886,7 +839,7 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     gaussjordan(a, b, 8);
 
     for (i = 0; i < 8; i++)
-        FREE(a[i]);
+        LEPT_FREE(a[i]);
 
     return 0;
 }
