@@ -195,7 +195,7 @@ PIXA      *pixa;
         /* Locate them; use small threshold for edges. */
     boxa = pixLocateBarcodes(pixs, 20, &pixb, &pixm);
     n = boxaGetCount(boxa);
-    L_INFO_INT("%d possible barcode(s) found", procName, n);
+    L_INFO("%d possible barcode(s) found\n", procName, n);
     if (n == 0) {
         boxaDestroy(&boxa);
         pixDestroy(&pixb);
@@ -214,12 +214,11 @@ PIXA      *pixa;
     for (i = 0; i < n; i++) {
         box = boxaGetBox(boxa, i, L_CLONE);
         pixt = pixDeskewBarcode(pixs, pixb, box, 15, 20, &angle, &conf);
-        L_INFO_FLOAT2("angle = %6.2f, conf = %6.2f", procName, angle, conf);
-	if (conf > 5.0) {
+        L_INFO("angle = %6.2f, conf = %6.2f\n", procName, angle, conf);
+        if (conf > 5.0) {
             pixaAddPix(pixa, pixt, L_INSERT);
             pixaAddBox(pixa, box, L_INSERT);
-        }
-        else {
+        } else {
             pixDestroy(&pixt);
             boxDestroy(&box);
         }
@@ -282,13 +281,13 @@ SARRAY    *saw, *sad;
         na = pixReadBarcodeWidths(pixt, method, debugflag);
         pixDestroy(&pixt);
         if (!na) {
-	    ERROR_INT("valid barcode widths not returned", procName, 1);
+            ERROR_INT("valid barcode widths not returned", procName, 1);
             continue;
-	}
+        }
 
             /* Save the widths as a string */
         nbars = numaGetCount(na);
-        barstr = (char *)CALLOC(nbars + 1, sizeof(char));
+        barstr = (char *)LEPT_CALLOC(nbars + 1, sizeof(char));
         for (j = 0; j < nbars; j++) {
             numaGetIValue(na, j, &ival);
             barstr[j] = 0x30 + ival;
@@ -299,10 +298,10 @@ SARRAY    *saw, *sad;
             /* Decode the width strings */
         data = barcodeDispatchDecoder(barstr, format, debugflag);
         if (!data) {
-	    ERROR_INT("barcode not decoded", procName, 1);
+            ERROR_INT("barcode not decoded", procName, 1);
             sarrayAddString(sad, emptystring, L_COPY);
             continue;
-	}
+        }
         sarrayAddString(sad, data, L_INSERT);
     }
 
@@ -534,30 +533,27 @@ PIX       *pixt1, *pixt2, *pixt3, *pixt4, *pixt5, *pixt6, *pixd;
         /* Because we're using the boundary pixels of the barcodes,
          * the peak can be sharper (and the confidence ratio higher)
          * from the signal across the top and bottom of the barcode.
-	 * However, the max score, which is the magnitude of the signal
-	 * at the optimum skew angle, will be smaller, so we use the
-	 * max score as the primary indicator of orientation. */
+         * However, the max score, which is the magnitude of the signal
+         * at the optimum skew angle, will be smaller, so we use the
+         * max score as the primary indicator of orientation. */
     if (score1 >= score2) {
         conf = conf1;
         if (conf1 > 6.0 && L_ABS(angle1) > 0.1) {
             angle = angle1;
             pixt5 = pixRotate(pixt2, deg2rad * angle1, L_ROTATE_AREA_MAP,
                               L_BRING_IN_WHITE, 0, 0);
-        }
-        else {
+        } else {
             angle = 0.0;
             pixt5 = pixClone(pixt2);
         }
-    }
-    else {  /* score2 > score1 */
+    } else {  /* score2 > score1 */
         conf = conf2;
         pixt6 = pixRotateOrth(pixt2, 1);
         if (conf2 > 6.0 && L_ABS(angle2) > 0.1) {
             angle = 90.0 + angle2;
             pixt5 = pixRotate(pixt6, deg2rad * angle2, L_ROTATE_AREA_MAP,
                               L_BRING_IN_WHITE, 0, 0);
-        }
-        else {
+        } else {
             angle = 90.0;
             pixt5 = pixClone(pixt6);
         }
@@ -569,10 +565,9 @@ PIX       *pixt1, *pixt2, *pixt3, *pixt4, *pixt5, *pixt6, *pixd;
         /* Extract barcode plus a margin around it */
     boxa = pixLocateBarcodes(pixt5, threshold, 0, 0);
     if ((n = boxaGetCount(boxa)) != 1) {
-        L_WARNING_INT("barcode mask in %d components", procName, n);
+        L_WARNING("barcode mask in %d components\n", procName, n);
         boxat = boxaSort(boxa, L_SORT_BY_AREA, L_SORT_DECREASING, NULL);
-    }
-    else {
+    } else {
         boxat = boxaCopy(boxa, L_CLONE);
     }
     boxt = boxaGetBox(boxat, 0, L_CLONE);
@@ -623,8 +618,8 @@ NUMA *
 pixExtractBarcodeWidths1(PIX      *pixs,
                         l_float32  thresh,
                         l_float32  binfract,
-			NUMA     **pnaehist,
-			NUMA     **pnaohist,
+                        NUMA     **pnaehist,
+                        NUMA     **pnaohist,
                         l_int32    debugflag)
 {
 NUMA  *nac, *nad;
@@ -778,8 +773,7 @@ NUMA       *nad;
         first = 0;
         last = h - 1;
         nscans = h;
-    }
-    else {
+    } else {
         first = (h - nscans) / 2;
         last = first + nscans - 1;
     }
@@ -793,7 +787,7 @@ NUMA       *nad;
         for (i = first; i <= last; i++) {
             line = data + i * wpl;
             val = GET_DATA_BYTE(line, j);
-	    array[j] += val;
+            array[j] += val;
         }
         array[j] = array[j] / (l_float32)nscans;
     }
@@ -896,7 +890,7 @@ NUMA      *naerange, *naorange, *naelut, *naolut, *nad;
     ned = numaGetCount(naedist);
     nod = numaGetCount(naodist);
     if (nod != ned - 1)
-        L_WARNING("ned != nod + 1", procName);
+        L_WARNING("ned != nod + 1\n", procName);
     factor = 1.0 / (binfract * minsize);  /* for converting units */
     for (i = 0; i < ned - 1; i++) {
         numaGetFValue(naedist, i, &val);
@@ -1011,8 +1005,8 @@ NUMA      *naedist, *naodist;
         return ERROR_INT("n < 2", procName, 1);
 
         /* Get numas of distances between crossings.  Separate these
-	 * into even (e.g., black) and odd (e.g., white) spans.
-	 * For barcodes, the black spans are 0, 2, etc.  These
+         * into even (e.g., black) and odd (e.g., white) spans.
+         * For barcodes, the black spans are 0, 2, etc.  These
          * distances are in pixel units.  */
     naedist = numaCreate(n / 2 + 1);
     naodist = numaCreate(n / 2);
@@ -1033,8 +1027,7 @@ NUMA      *naedist, *naodist;
     numaGetMax(naedist, &maxdist, NULL);
     numaGetMax(naodist, &dist, NULL);
     maxdist = L_MAX(dist, maxdist);
-    L_INFO_FLOAT2("mindist = %7.3f, maxdist = %7.3f\n",
-                  procName, mindist, maxdist);
+    L_INFO("mindist = %7.3f, maxdist = %7.3f\n", procName, mindist, maxdist);
 
     if (pnaedist)
         *pnaedist = naedist;
@@ -1100,16 +1093,14 @@ NUMA      *nad;
         if (inpeak == FALSE && val > maxmin) {
             inpeak = TRUE;
             left = i;
-        }
-        else if (inpeak == TRUE && val <= maxmin) {  /* end peak */
+        } else if (inpeak == TRUE && val <= maxmin) {  /* end peak */
             center = (left + i - 1.0) / 2.0;
             if (center - prevcenter >= minsep) {  /* save new peak */
                 inpeak = FALSE;
                 numaAddNumber(nad, left);
                 numaAddNumber(nad, i - 1);
                 prevcenter = center;
-            }
-            else {  /* attach to previous peak; revise the right edge */
+            } else {  /* attach to previous peak; revise the right edge */
                 numaSetValue(nad, numaGetCount(nad) - 1, i - 1);
             }
         }
@@ -1135,7 +1126,7 @@ static NUMA *
 numaGetPeakCentroids(NUMA  *nahist,
                      NUMA  *narange)
 {
-l_int32    i, j, nh, nr, low, high;
+l_int32    i, j, nr, low, high;
 l_float32  cent, sum, val;
 NUMA      *nad;
 
@@ -1145,7 +1136,6 @@ NUMA      *nad;
         return (NUMA *)ERROR_PTR("nahist not defined", procName, NULL);
     if (!narange)
         return (NUMA *)ERROR_PTR("narange not defined", procName, NULL);
-    nh = numaGetCount(nahist);
     nr = numaGetCount(narange) / 2;
 
     nad = numaCreate(4);
@@ -1202,9 +1192,9 @@ NUMA       *nalut;
         return (NUMA *)ERROR_PTR("nc must be 1, 2, 3, or 4", procName, NULL);
 
         /* Check the peak centroids for consistency with bar widths.
-	 * The third peak can correspond to a width of either 3 or 4.
-	 * Use ratios 3/2 and 4/2 instead of 3/1 and 4/1 because the
-	 * former are more stable and closer to the expected ratio.  */
+         * The third peak can correspond to a width of either 3 or 4.
+         * Use ratios 3/2 and 4/2 instead of 3/1 and 4/1 because the
+         * former are more stable and closer to the expected ratio.  */
     if (nc > 1) {
         warray = numaGetFArray(nacent, L_NOCOPY);
         if (warray[0] == 0)
@@ -1212,16 +1202,16 @@ NUMA       *nalut;
                                      procName, NULL);
         rat21 = warray[1] / warray[0];
         if (rat21 < 1.5 || rat21 > 2.6)
-            L_WARNING_FLOAT("width ratio 2/1 = %f", procName, rat21);
+            L_WARNING("width ratio 2/1 = %f\n", procName, rat21);
         if (nc > 2) {
             rat32 = warray[2] / warray[1];
             if (rat32 < 1.3 || rat32 > 2.25)
-                L_WARNING_FLOAT("width ratio 3/2 = %f", procName, rat32);
+                L_WARNING("width ratio 3/2 = %f\n", procName, rat32);
         }
         if (nc == 4) {
             rat42 = warray[3] / warray[1];
             if (rat42 < 1.7 || rat42 > 2.3)
-                L_WARNING_FLOAT("width ratio 4/2 = %f", procName, rat42);
+                L_WARNING("width ratio 4/2 = %f\n", procName, rat42);
         }
     }
 
@@ -1273,10 +1263,10 @@ NUMA       *nalut;
  */
 NUMA *
 numaQuantizeCrossingsByWindow(NUMA       *nas,
-		              l_float32   ratio,
+                              l_float32   ratio,
                               l_float32  *pwidth,
                               l_float32  *pfirstloc,
-			      NUMA      **pnac,
+                              NUMA      **pnac,
                               l_int32     debugflag)
 {
 l_int32    i, nw, started, count, trans;
@@ -1303,8 +1293,8 @@ NUMA      *nac, *nad;
     numaEvalBestWidthAndShift(nas, 100, 10, 0.98 * minwidth, 1.02 * minwidth,
                               &minwidth, &minshift, NULL);
 
-    L_INFO_FLOAT2("best width = %7.3f, best shift = %7.3f\n",
-                  procName, minwidth, minshift);
+    L_INFO("best width = %7.3f, best shift = %7.3f\n",
+           procName, minwidth, minshift);
 
         /* Get the crossing array (0,1,2) for the best window width and shift */
     numaEvalSyncError(nas, 0, 0, minwidth, minshift, NULL, &nac);
@@ -1321,20 +1311,20 @@ NUMA      *nac, *nad;
     count = 0;  /* unnecessary init */
     for (i = 0; i < nw; i++) {
         numaGetIValue(nac, i, &trans);
-	if (trans > 2)
-            L_WARNING_INT("trans = %d > 2 !!!", procName, trans);
+        if (trans > 2)
+            L_WARNING("trans = %d > 2 !!!\n", procName, trans);
         if (started) {
             if (trans > 1) {  /* i.e., when trans == 2 */
                 numaAddNumber(nad, count);
                 trans--;
-		count = 1;
-            }
-	    if (trans == 1) {
-                numaAddNumber(nad, count);
                 count = 1;
             }
-            else
+            if (trans == 1) {
+                numaAddNumber(nad, count);
+                count = 1;
+            } else {
                 count++;
+            }
         }
         if (!started && trans) {
             started = TRUE;
@@ -1468,7 +1458,7 @@ NUMA      *nad;
     nc = ilast - ifirst + 1;
 
         /* Set up an array corresponding to the (shifted) windows,
-	 * and fill in the crossings. */
+         * and fill in the crossings. */
     score = 0.0;
     numaGetFValue(nas, ifirst, &xfirst);
     numaGetFValue(nas, ilast, &xlast);
@@ -1478,11 +1468,11 @@ NUMA      *nad;
     xleft = xfirst - width / 2.0 + shift;  /* left edge of first window */
     for (i = ifirst; i <= ilast; i++) {
         numaGetFValue(nas, i, &xc);
-	iw = (l_int32)((xc - xleft) / width);
-	xwc = xleft + (iw + 0.5) * width;  /* center of cell iw */
-	score += (xwc - xc) * (xwc - xc);
-	numaGetIValue(nad, iw, &ival);
-	numaSetValue(nad, iw, ival + 1);
+        iw = (l_int32)((xc - xleft) / width);
+        xwc = xleft + (iw + 0.5) * width;  /* center of cell iw */
+        score += (xwc - xc) * (xwc - xc);
+        numaGetIValue(nad, iw, &ival);
+        numaSetValue(nad, iw, ival + 1);
     }
 
     if (pscore)
