@@ -55,8 +55,8 @@
  *          char      *sarrayToString()
  *          char      *sarrayToStringRange()
  *
- *      Concatenate 2 sarrays
- *          l_int32    sarrayConcatenate()
+ *      Join 2 sarrays
+ *          l_int32    sarrayJoin()
  *          l_int32    sarrayAppendRange()
  *
  *      Pad an sarray to be the same size as another sarray
@@ -78,6 +78,18 @@
  *          SARRAY    *sarraySortByIndex()
  *          l_int32    stringCompareLexical()
  *
+ *      Operations by aset (rbtree)
+ *          SARRAY    *sarrayUnionByAset()
+ *          SARRAY    *sarrayRemoveDupsByAset()
+ *          SARRAY    *sarrayIntersectionByAset()
+ *          L_ASET    *l_asetCreateFromSarray()
+ *
+ *      Operations by hashmap (dnahash)
+ *          l_int32    sarrayRemoveDupsByHash()
+ *          SARRAY     *sarrayIntersectionByHash()
+ *          l_int32     sarrayFindStringByHash()
+ *          L_DNAHASH  *l_dnaHashCreateFromSarray()
+ *
  *      Serialize for I/O
  *          SARRAY    *sarrayRead()
  *          SARRAY    *sarrayReadStream()
@@ -143,7 +155,7 @@
 static const l_int32  INITIAL_PTR_ARRAYSIZE = 50;     /* n'importe quoi */
 static const l_int32  L_BUF_SIZE = 512;
 
-    /* Static function */
+    /* Static functions */
 static l_int32 sarrayExtendArray(SARRAY *sa);
 
 
@@ -167,9 +179,9 @@ SARRAY  *sa;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((sa = (SARRAY *)CALLOC(1, sizeof(SARRAY))) == NULL)
+    if ((sa = (SARRAY *)LEPT_CALLOC(1, sizeof(SARRAY))) == NULL)
         return (SARRAY *)ERROR_PTR("sa not made", procName, NULL);
-    if ((sa->array = (char **)CALLOC(n, sizeof(char *))) == NULL)
+    if ((sa->array = (char **)LEPT_CALLOC(n, sizeof(char *))) == NULL)
         return (SARRAY *)ERROR_PTR("ptr array not made", procName, NULL);
 
     sa->nalloc = n;
@@ -266,8 +278,8 @@ SARRAY  *sa;
  *      (2) The newline characters are removed from each substring.
  */
 SARRAY *
-sarrayCreateLinesFromString(char    *string,
-                            l_int32  blankflag)
+sarrayCreateLinesFromString(const char  *string,
+                            l_int32      blankflag)
 {
 l_int32  i, nsub, size, startptr;
 char    *cstring, *substring;
@@ -315,7 +327,7 @@ SARRAY  *sa;
             sarrayAddString(sa, substring, L_INSERT);
 /*            fprintf(stderr, "substring = %s\n", substring); */
         }
-        FREE(cstring);
+        LEPT_FREE(cstring);
     } else {  /* remove blank lines; use strtok */
         sarraySplitString(sa, string, "\r\n");
     }
@@ -354,11 +366,11 @@ SARRAY  *sa;
         if (sa->array) {
             for (i = 0; i < sa->n; i++) {
                 if (sa->array[i])
-                    FREE(sa->array[i]);
+                    LEPT_FREE(sa->array[i]);
             }
-            FREE(sa->array);
+            LEPT_FREE(sa->array);
         }
-        FREE(sa);
+        LEPT_FREE(sa);
     }
 
     *psa = NULL;
@@ -557,7 +569,7 @@ l_int32  n;
     if (copyflag != L_INSERT && copyflag != L_COPY)
         return ERROR_INT("invalid copyflag", procName, 1);
 
-    FREE(sa->array[index]);
+    LEPT_FREE(sa->array[index]);
     if (copyflag == L_INSERT)
         str = newstr;
     else  /* L_COPY */
@@ -583,7 +595,7 @@ l_int32  i;
     if (!sa)
         return ERROR_INT("sa not defined", procName, 1);
     for (i = 0; i < sa->n; i++) {  /* free strings and null ptrs */
-        FREE(sa->array[i]);
+        LEPT_FREE(sa->array[i]);
         sa->array[i] = NULL;
     }
     sa->n = 0;
@@ -821,7 +833,7 @@ l_int32  n, i, last, size, index, len;
         size += strlen(str) + 2;
     }
 
-    if ((dest = (char *)CALLOC(size + 1, sizeof(char))) == NULL)
+    if ((dest = (char *)LEPT_CALLOC(size + 1, sizeof(char))) == NULL)
         return (char *)ERROR_PTR("dest not made", procName, NULL);
 
     index = 0;
@@ -844,10 +856,10 @@ l_int32  n, i, last, size, index, len;
 
 
 /*----------------------------------------------------------------------*
- *                      Concatenate 2 sarrays                           *
+ *                           Join 2 sarrays                             *
  *----------------------------------------------------------------------*/
 /*!
- *  sarrayConcatenate()
+ *  sarrayJoin()
  *
  *      Input:  sa1  (to be added to)
  *              sa2  (append to sa1)
@@ -857,13 +869,13 @@ l_int32  n, i, last, size, index, len;
  *      (1) Copies of the strings in sarray2 are added to sarray1.
  */
 l_int32
-sarrayConcatenate(SARRAY  *sa1,
-                  SARRAY  *sa2)
+sarrayJoin(SARRAY  *sa1,
+           SARRAY  *sa2)
 {
 char    *str;
 l_int32  n, i;
 
-    PROCNAME("sarrayConcatenate");
+    PROCNAME("sarrayJoin");
 
     if (!sa1)
         return ERROR_INT("sa1 not defined", procName, 1);
@@ -1098,7 +1110,7 @@ char  *cstr, *substr, *saveptr;
         sarrayAddString(sa, substr, L_INSERT);
     while ((substr = strtokSafe(NULL, separators, &saveptr)))
         sarrayAddString(sa, substr, L_INSERT);
-    FREE(cstr);
+    LEPT_FREE(cstr);
 
     return 0;
 }
@@ -1444,6 +1456,408 @@ l_int32  i, len1, len2, len;
 }
 
 
+/*----------------------------------------------------------------------*
+ *                       Operations by aset (rbtree)                    *
+ *----------------------------------------------------------------------*/
+/*!
+ *  sarrayUnionByAset()
+ *
+ *      Input:  sa1, sa2
+ *      Return: sad (with the union of the string set), or null on error
+ *
+ *  Notes:
+ *      (1) Duplicates are removed from the concatenation of the two arrays.
+ *      (2) The key for each string is a 64-bit hash.
+ *      (2) Algorithm: Concatenate the two sarrays.  Then build a set,
+ *          using hashed strings as keys.  As the set is built, first do
+ *          a find; if not found, add the key to the set and add the string
+ *          to the output sarray.  This is O(nlogn).
+ */
+SARRAY *
+sarrayUnionByAset(SARRAY  *sa1,
+                  SARRAY  *sa2)
+{
+SARRAY  *sa3, *sad;
+
+    PROCNAME("sarrayUnionByAset");
+
+    if (!sa1)
+        return (SARRAY *)ERROR_PTR("sa1 not defined", procName, NULL);
+    if (!sa2)
+        return (SARRAY *)ERROR_PTR("sa2 not defined", procName, NULL);
+
+        /* Join */
+    sa3 = sarrayCopy(sa1);
+    sarrayJoin(sa3, sa2);
+
+        /* Eliminate duplicates */
+    sad = sarrayRemoveDupsByAset(sa3);
+    sarrayDestroy(&sa3);
+    return sad;
+}
+
+
+/*!
+ *  sarrayRemoveDupsByAset()
+ *
+ *      Input:  sas
+ *      Return: sad (with duplicates removed), or null on error
+ *
+ *  Notes:
+ *      (1) This is O(nlogn), considerably slower than
+ *          sarrayRemoveDupsByHash() for large string arrays.
+ *      (2) The key for each string is a 64-bit hash.
+ *      (3) Build a set, using hashed strings as keys.  As the set is
+ *          built, first do a find; if not found, add the key to the
+ *          set and add the string to the output sarray.
+ */
+SARRAY *
+sarrayRemoveDupsByAset(SARRAY  *sas)
+{
+char     *str;
+l_int32   i, n;
+l_uint64  hash;
+L_ASET   *set;
+RB_TYPE   key;
+SARRAY   *sad;
+
+    PROCNAME("sarrayRemoveDupsByAset");
+
+    if (!sas)
+        return (SARRAY *)ERROR_PTR("sas not defined", procName, NULL);
+
+    set = l_asetCreate(L_UINT_TYPE);
+    sad = sarrayCreate(0);
+    n = sarrayGetCount(sas);
+    for (i = 0; i < n; i++) {
+        str = sarrayGetString(sas, i, L_NOCOPY);
+        l_hashStringToUint64(str, &hash);
+        key.utype = hash;
+        if (!l_asetFind(set, key)) {
+            sarrayAddString(sad, str, L_COPY);
+            l_asetInsert(set, key);
+        }
+    }
+
+    l_asetDestroy(&set);
+    return sad;
+}
+
+
+/*!
+ *  sarrayIntersectionByAset()
+ *
+ *      Input:  sa1, sa2
+ *      Return: sad (with the intersection of the string set), or null on error
+ *
+ *  Notes:
+ *      (1) Algorithm: put the smaller sarray into a set, using the string
+ *          hashes as the key values.  Then run through the larger sarray,
+ *          building an output sarray and a second set from the strings
+ *          in the larger array: if a string is in the first set but
+ *          not in the second, add the string to the output sarray and hash
+ *          it into the second set.  The second set is required to make
+ *          sure only one instance of each string is put into the output sarray.
+ *          This is O(mlogn), {m,n} = sizes of {smaller,larger} input arrays.
+ */
+SARRAY *
+sarrayIntersectionByAset(SARRAY  *sa1,
+                         SARRAY  *sa2)
+{
+char     *str;
+l_int32   n1, n2, i, n;
+l_uint64  hash;
+L_ASET   *set1, *set2;
+RB_TYPE   key;
+SARRAY   *sa_small, *sa_big, *sad;
+
+    PROCNAME("sarrayIntersectionByAset");
+
+    if (!sa1)
+        return (SARRAY *)ERROR_PTR("sa1 not defined", procName, NULL);
+    if (!sa2)
+        return (SARRAY *)ERROR_PTR("sa2 not defined", procName, NULL);
+
+        /* Put the elements of the biggest array into a set */
+    n1 = sarrayGetCount(sa1);
+    n2 = sarrayGetCount(sa2);
+    sa_small = (n1 < n2) ? sa1 : sa2;   /* do not destroy sa_small */
+    sa_big = (n1 < n2) ? sa2 : sa1;   /* do not destroy sa_big */
+    set1 = l_asetCreateFromSarray(sa_big);
+
+        /* Build up the intersection of strings */
+    sad = sarrayCreate(0);
+    n = sarrayGetCount(sa_small);
+    set2 = l_asetCreate(L_UINT_TYPE);
+    for (i = 0; i < n; i++) {
+        str = sarrayGetString(sa_small, i, L_NOCOPY);
+        l_hashStringToUint64(str, &hash);
+        key.utype = hash;
+        if (l_asetFind(set1, key) && !l_asetFind(set2, key)) {
+            sarrayAddString(sad, str, L_COPY);
+            l_asetInsert(set2, key);
+        }
+    }
+
+    l_asetDestroy(&set1);
+    l_asetDestroy(&set2);
+    return sad;
+}
+
+
+/*!
+ *  l_asetCreateFromSarray()
+ *
+ *      Input:  sa
+ *      Return: set (using a string hash into a uint32 as the key)
+ */
+L_ASET *
+l_asetCreateFromSarray(SARRAY  *sa)
+{
+char     *str;
+l_int32   i, n;
+l_uint64  hash;
+L_ASET   *set;
+RB_TYPE   key;
+
+    PROCNAME("l_asetCreateFromSarray");
+
+    if (!sa)
+        return (L_ASET *)ERROR_PTR("sa not defined", procName, NULL);
+
+    set = l_asetCreate(L_UINT_TYPE);
+    n = sarrayGetCount(sa);
+    for (i = 0; i < n; i++) {
+        str = sarrayGetString(sa, i, L_NOCOPY);
+        l_hashStringToUint64(str, &hash);
+        key.utype = hash;
+        l_asetInsert(set, key);
+    }
+
+    return set;
+}
+
+
+/*----------------------------------------------------------------------*
+ *                    Operations by hashmap (dnahash)                   *
+ *----------------------------------------------------------------------*/
+/*!
+ *  sarrayRemoveDupsByHash()
+ *
+ *      Input:  sas
+ *              &sad (<return> unique set of strings; duplicates removed)
+ *              &dahash (<optional return> dnahash used for lookup)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Generates a sarray with unique values.
+ *      (2) The dnahash is built up with sad to assure uniqueness.
+ *          It can be used to find if a string is in the set:
+ *              sarrayFindValByHash(sad, dahash, str, &index)
+ *      (3) The hash of the string location is simple and fast.  It scales
+ *          up with the number of buckets to insure a fairly random
+ *          bucket selection input strings.
+ *      (4) This is faster than sarrayRemoveDupsByAset(), because the
+ *          bucket lookup is O(n), although there is a double-loop
+ *          lookup within the dna in each bucket.
+ */
+l_int32
+sarrayRemoveDupsByHash(SARRAY      *sas,
+                       SARRAY     **psad,
+                       L_DNAHASH  **pdahash)
+{
+char       *str;
+l_int32     i, n, index, items;
+l_uint32    nsize;
+l_uint64    key;
+l_float64   val;
+SARRAY     *sad;
+L_DNAHASH  *dahash;
+
+    PROCNAME("sarrayRemoveDupsByHash");
+
+    if (pdahash) *pdahash = NULL;
+    if (!psad)
+        return ERROR_INT("&sad not defined", procName, 1);
+    *psad = NULL;
+    if (!sas)
+        return ERROR_INT("sas not defined", procName, 1);
+
+    n = sarrayGetCount(sas);
+    findNextLargerPrime(n / 20, &nsize);  /* buckets in hash table */
+    dahash = l_dnaHashCreate(nsize, 8);
+    sad = sarrayCreate(n);
+    *psad = sad;
+    for (i = 0, items = 0; i < n; i++) {
+        str = sarrayGetString(sas, i, L_NOCOPY);
+        sarrayFindStringByHash(sad, dahash, str, &index);
+        if (index < 0) {  /* not found */
+            l_hashStringToUint64(str, &key);
+            l_dnaHashAdd(dahash, key, (l_float64)items);
+            sarrayAddString(sad, str, L_COPY);
+            items++;
+        }
+    }
+
+    if (pdahash)
+        *pdahash = dahash;
+    else
+        l_dnaHashDestroy(&dahash);
+    return 0;
+}
+
+
+/*!
+ *  sarrayIntersectionByHash()
+ *
+ *      Input:  sa1, sa2
+ *      Return: sad (intersection of the strings), or null on error
+ *
+ *  Notes:
+ *      (1) This is faster than sarrayIntersectionByAset(), because the
+ *          bucket lookup is O(n).
+ */
+SARRAY *
+sarrayIntersectionByHash(SARRAY  *sa1,
+                         SARRAY  *sa2)
+{
+char       *str;
+l_int32     n1, n2, nsmall, i, index1, index2;
+l_uint32    nsize2;
+l_uint64    key;
+L_DNAHASH  *dahash1, *dahash2;
+SARRAY     *sa_small, *sa_big, *sad;
+
+    PROCNAME("sarrayIntersectionByHash");
+
+    if (!sa1)
+        return (SARRAY *)ERROR_PTR("sa1 not defined", procName, NULL);
+    if (!sa2)
+        return (SARRAY *)ERROR_PTR("sa2 not defined", procName, NULL);
+
+        /* Put the elements of the biggest sarray into a dnahash */
+    n1 = sarrayGetCount(sa1);
+    n2 = sarrayGetCount(sa2);
+    sa_small = (n1 < n2) ? sa1 : sa2;   /* do not destroy sa_small */
+    sa_big = (n1 < n2) ? sa2 : sa1;   /* do not destroy sa_big */
+    dahash1 = l_dnaHashCreateFromSarray(sa_big);
+
+        /* Build up the intersection of strings.  Add to @sad
+         * if the string is in sa_big (using dahash1) but hasn't
+         * yet been seen in the traversal of sa_small (using dahash2). */
+    sad = sarrayCreate(0);
+    nsmall = sarrayGetCount(sa_small);
+    findNextLargerPrime(nsmall / 20, &nsize2);  /* buckets in hash table */
+    dahash2 = l_dnaHashCreate(nsize2, 0);
+    for (i = 0; i < nsmall; i++) {
+        str = sarrayGetString(sa_small, i, L_NOCOPY);
+        sarrayFindStringByHash(sa_big, dahash1, str, &index1);
+        if (index1 >= 0) {
+            sarrayFindStringByHash(sa_small, dahash2, str, &index2);
+            if (index2 == -1) {
+                sarrayAddString(sad, str, L_COPY);
+                l_hashStringToUint64(str, &key);
+                l_dnaHashAdd(dahash2, key, (l_float64)i);
+            }
+        }
+    }
+
+    l_dnaHashDestroy(&dahash1);
+    l_dnaHashDestroy(&dahash2);
+    return sad;
+}
+
+
+/*!
+ *  sarrayFindStringByHash()
+ *
+ *      Input:  sa
+ *              dahash (built from sa)
+ *              str  (arbitrary string)
+ *              &index (<return> index into @sa if @str is in @sa;
+ *              -1 otherwise)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Fast lookup in dnaHash associated with a sarray, to see if a
+ *          random string @str is already stored in the hash table.
+ */
+l_int32
+sarrayFindStringByHash(SARRAY      *sa,
+                       L_DNAHASH   *dahash,
+                       const char  *str,
+                       l_int32     *pindex)
+{
+char     *stri;
+l_int32   i, nvals, index;
+l_uint64  key;
+L_DNA    *da;
+
+    PROCNAME("sarrayFindStringByHash");
+
+    if (!pindex)
+        return ERROR_INT("&index not defined", procName, 1);
+    *pindex = -1;
+    if (!sa)
+        return ERROR_INT("sa not defined", procName, 1);
+    if (!dahash)
+        return ERROR_INT("dahash not defined", procName, 1);
+
+    l_hashStringToUint64(str, &key);
+    da = l_dnaHashGetDna(dahash, key, L_NOCOPY);
+    if (!da) return 0;
+
+        /* Run through the da, looking for this string */
+    nvals = l_dnaGetCount(da);
+    for (i = 0; i < nvals; i++) {
+        l_dnaGetIValue(da, i, &index);
+        stri = sarrayGetString(sa, index, L_NOCOPY);
+        if (!strcmp(str, stri)) {  /* duplicate */
+            *pindex = index;
+            return 0;
+        }
+    }
+
+    return 0;
+}
+
+
+/*!
+ *  l_dnaHashCreateFromSarray()
+ *
+ *      Input:  sa
+ *      Return: dahash, or null on error
+ */
+L_DNAHASH *
+l_dnaHashCreateFromSarray(SARRAY  *sa)
+{
+char       *str;
+l_int32     i, n;
+l_uint32    nsize;
+l_uint64    key;
+L_DNAHASH  *dahash;
+
+        /* Build up dnaHash of indices, hashed by a 64-bit key that
+         * should randomize the lower bits used in bucket selection.
+         * Having about 20 pts in each bucket is roughly optimal. */
+    n = sarrayGetCount(sa);
+    findNextLargerPrime(n / 20, &nsize);  /* buckets in hash table */
+/*    fprintf(stderr, "Prime used: %d\n", nsize); */
+
+        /* Add each string, using the hash as key and the index into @sa
+         * as the value.  Storing the index enables operations that check
+         * for duplicates.  */
+    dahash = l_dnaHashCreate(nsize, 8);
+    for (i = 0; i < n; i++) {
+        str = sarrayGetString(sa, i, L_NOCOPY);
+        l_hashStringToUint64(str, &key);
+        l_dnaHashAdd(dahash, key, (l_float64)i);
+    }
+
+    return dahash;
+}
+
+
 /*----------------------------------------------------------------------*
  *                           Serialize for I/O                          *
  *----------------------------------------------------------------------*/
@@ -1511,7 +1925,7 @@ SARRAY  *sa;
     if ((sa = sarrayCreate(n)) == NULL)
         return (SARRAY *)ERROR_PTR("sa not made", procName, NULL);
     bufsize = L_BUF_SIZE + 1;
-    if ((stringbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL)
+    if ((stringbuf = (char *)LEPT_CALLOC(bufsize, sizeof(char))) == NULL)
         return (SARRAY *)ERROR_PTR("stringbuf not made", procName, NULL);
 
     for (i = 0; i < n; i++) {
@@ -1520,9 +1934,9 @@ SARRAY  *sa;
             return (SARRAY *)ERROR_PTR("error on string size", procName, NULL);
             /* Expand the string buffer if necessary */
         if (size > bufsize - 5) {
-            FREE(stringbuf);
+            LEPT_FREE(stringbuf);
             bufsize = (l_int32)(1.5 * size);
-            stringbuf = (char *)CALLOC(bufsize, sizeof(char));
+            stringbuf = (char *)LEPT_CALLOC(bufsize, sizeof(char));
         }
             /* Read the stored string, plus leading spaces and trailing \n */
         if (fread(stringbuf, 1, size + 3, fp) != size + 3)
@@ -1534,7 +1948,7 @@ SARRAY  *sa;
     }
     ignore = fscanf(fp, "\n");
 
-    FREE(stringbuf);
+    LEPT_FREE(stringbuf);
     return sa;
 }
 
@@ -1668,7 +2082,7 @@ FILE  *fp;
  *      (4) It is assumed that the page number is contained within
  *          the basename (the filename without directory or extension).
  *          @numpre is the number of characters in the basename
- *          preceeding the actual page number; @numpost is the number
+ *          preceding the actual page number; @numpost is the number
  *          following the page number, up to either the end of the
  *          basename or a ".", whichever comes first.
  *      (5) This is useful when all filenames contain numbers that are
@@ -1876,7 +2290,7 @@ struct dirent  *pdirentry;
 
     realdir = genPathname(dirname, NULL);
     pdir = opendir(realdir);
-    FREE(realdir);
+    LEPT_FREE(realdir);
     if (!pdir)
         return (SARRAY *)ERROR_PTR("pdir not opened", procName, NULL);
     if ((safiles = sarrayCreate(0)) == NULL)
@@ -1924,22 +2338,22 @@ WIN32_FIND_DATAA  ffd;
 
     realdir = genPathname(dirname, NULL);
     pszDir = stringJoin(realdir, "\\*");
-    FREE(realdir);
+    LEPT_FREE(realdir);
 
     if (strlen(pszDir) + 1 > MAX_PATH) {
-        FREE(pszDir);
+        LEPT_FREE(pszDir);
         return (SARRAY *)ERROR_PTR("dirname is too long", procName, NULL);
     }
 
     if ((safiles = sarrayCreate(0)) == NULL) {
-        FREE(pszDir);
+        LEPT_FREE(pszDir);
         return (SARRAY *)ERROR_PTR("safiles not made", procName, NULL);
     }
 
     hFind = FindFirstFileA(pszDir, &ffd);
     if (INVALID_HANDLE_VALUE == hFind) {
         sarrayDestroy(&safiles);
-        FREE(pszDir);
+        LEPT_FREE(pszDir);
         return (SARRAY *)ERROR_PTR("hFind not opened", procName, NULL);
     }
 
@@ -1951,7 +2365,7 @@ WIN32_FIND_DATAA  ffd;
     }
 
     FindClose(hFind);
-    FREE(pszDir);
+    LEPT_FREE(pszDir);
     return safiles;
 }
 
